
/*

Copyright (c) Future Technology Devices International 2014

THIS SOFTWARE IS PROVIDED BY FUTURE TECHNOLOGY DEVICES INTERNATIONAL LIMITED "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
FUTURE TECHNOLOGY DEVICES INTERNATIONAL LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES LOSS OF USE, DATA, OR PROFITS OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

FTDI DRIVERS MAY BE USED ONLY IN CONJUNCTION WITH PRODUCTS BASED ON FTDI PARTS.

FTDI DRIVERS MAY BE DISTRIBUTED IN ANY FORM AS LONG AS LICENSE INFORMATION IS NOT MODIFIED.

IF A CUSTOM VENDOR ID AND/OR PRODUCT ID OR DESCRIPTION STRING ARE USED, IT IS THE
RESPONSIBILITY OF THE PRODUCT MANUFACTURER TO MAINTAIN ANY CHANGES AND SUBSEQUENT WHQL
RE-CERTIFICATION AS A RESULT OF MAKING THESE CHANGES.

Abstract:

This file contains is functions for all UI fields.

Author : FTDI */


#include <stdio.h>
#include <Arduino.h>
#include <EEPROM.h>
#include <SPI.h>
#include <avr/pgmspace.h>




/* Sample application to demonstrat FT800 primitives, widgets and customized screen shots */

#include "FT_Platform.h"
#include "sdcard.h"


#define SAMAPP_DELAY_BTW_APIS (1000)
#define SAMAPP_ENABLE_DELAY() Ft_Gpu_Hal_Sleep(SAMAPP_DELAY_BTW_APIS)
#define SAMAPP_ENABLE_DELAY_VALUE(x) Ft_Gpu_Hal_Sleep(x)
#define F16(s)        ((ft_int32_t)((s) * 65536))
#define SQ(v) (v*v)
#define MEMCMD(a) Ft_Gpu_Hal_WrCmdBuf(phost,a,sizeof(a))

/* Global variables for display resolution to support various display panels */
/* Default is WQVGA - 480x272 */
ft_int16_t FT_DispWidth = 480;
ft_int16_t FT_DispHeight = 272;
ft_int16_t FT_DispHCycle =  548;
ft_int16_t FT_DispHOffset = 43;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 41;
ft_int16_t FT_DispVCycle = 292;
ft_int16_t FT_DispVOffset = 12;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 10;
ft_uint8_t FT_DispPCLK = 5;
ft_char8_t FT_DispSwizzle = 0;
ft_char8_t FT_DispPCLKPol = 1;
ft_char8_t FT_DispCSpread = 1;
ft_char8_t FT_DispDither = 1;

/* Global used for buffer optimization */
Ft_Gpu_Hal_Context_t host,*phost;


ft_int16_t CartX = 480;

ft_uint32_t Ft_CmdBuffer_Index;
ft_uint32_t Ft_DlBuffer_Index;

#ifdef BUFFER_OPTIMIZATION
ft_uint8_t  Ft_DlBuffer[FT_DL_SIZE];
ft_uint8_t  Ft_CmdBuffer[FT_CMD_FIFO_SIZE];
#endif

/* Initial boot up DL - make the back ground green color */
const ft_uint8_t FT_DLCODE_BOOTUP[12] =
{
  255,255,255,2,//GPU instruction CLEAR_COLOR_RGB
  7,0,0,38, //GPU instruction CLEAR
  0,0,0,0,  //GPU instruction DISPLAY
};


ft_void_t Ft_App_WrCoCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef  BUFFER_OPTIMIZATION
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_CmdBuffer[Ft_CmdBuffer_Index];
   *pBuffcmd = cmd;
#endif
#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif

   /* Increment the command index */
   Ft_CmdBuffer_Index += FT_CMD_SIZE;  
}

ft_void_t Ft_App_WrDlCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef BUFFER_OPTIMIZATION  
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_DlBuffer[Ft_DlBuffer_Index];
   *pBuffcmd = cmd;
#endif

#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
   /* Increment the command index */
   Ft_DlBuffer_Index += FT_CMD_SIZE;  
}

ft_void_t Ft_App_WrCoStr_Buffer(Ft_Gpu_Hal_Context_t *phost,const ft_char8_t *s)
{
#ifdef  BUFFER_OPTIMIZATION  
  ft_uint16_t length = 0;
  length = strlen(s) + 1;//last for the null termination
  
  strcpy(&Ft_CmdBuffer[Ft_CmdBuffer_Index],s);  

  /* increment the length and align it by 4 bytes */
  Ft_CmdBuffer_Index += ((length + 3) & ~3);  
#endif  
}

ft_void_t Ft_App_Flush_DL_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_DlBuffer_Index> 0)
     Ft_Gpu_Hal_WrMem(phost,RAM_DL,Ft_DlBuffer,Ft_DlBuffer_Index);
#endif     
   Ft_DlBuffer_Index = 0;
   
}

ft_void_t Ft_App_Flush_Co_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_CmdBuffer_Index > 0)
     Ft_Gpu_Hal_WrCmdBuf(phost,Ft_CmdBuffer,Ft_CmdBuffer_Index);
#endif     
   Ft_CmdBuffer_Index = 0;
}


/* API to give fadeout effect by changing the display PWM from 100 till 0 */
ft_void_t SAMAPP_fadeout()
{
   ft_int32_t i;
	
	for (i = 100; i >= 0; i -= 3) 
	{
		Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);

		Ft_Gpu_Hal_Sleep(2);//sleep for 2 ms
	}
}

/* API to perform display fadein effect by changing the display PWM from 0 till 100 and finally 128 */
ft_void_t SAMAPP_fadein()
{
	ft_int32_t i;
	
	for (i = 0; i <=100 ; i += 3) 
	{
		Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);
		Ft_Gpu_Hal_Sleep(2);//sleep for 2 ms
	}
	/* Finally make the PWM 100% */
	i = 128;
	Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);
}


/* API to check the status of previous DLSWAP and perform DLSWAP of new DL */
/* Check for the status of previous DLSWAP and if still not done wait for few ms and check again */
ft_void_t SAMAPP_GPU_DLSwap(ft_uint8_t DL_Swap_Type)
{
	ft_uint8_t Swap_Type = DLSWAP_FRAME,Swap_Done = DLSWAP_FRAME;

	if(DL_Swap_Type == DLSWAP_LINE)
	{
		Swap_Type = DLSWAP_LINE;
	}

	/* Perform a new DL swap */
	Ft_Gpu_Hal_Wr8(phost,REG_DLSWAP,Swap_Type);

	/* Wait till the swap is done */
	while(Swap_Done)
	{
		Swap_Done = Ft_Gpu_Hal_Rd8(phost,REG_DLSWAP);

		if(DLSWAP_DONE != Swap_Done)
		{
			Ft_Gpu_Hal_Sleep(10);//wait for 10ms
		}
	}	
}


ft_void_t Ft_BootupConfig()
{
	Ft_Gpu_Hal_Powercycle(phost,FT_TRUE);

		/* Access address 0 to wake up the FT800 */
		Ft_Gpu_HostCommand(phost,FT_GPU_ACTIVE_M);
		Ft_Gpu_Hal_Sleep(20);

		/* Set the clk to external clock */
#if (!defined(ME800A_HV35R) && !defined(ME810A_HV35R))
		Ft_Gpu_HostCommand(phost,FT_GPU_EXTERNAL_OSC);
		Ft_Gpu_Hal_Sleep(10);
#endif

		{
			ft_uint8_t chipid;
			//Read Register ID to check if FT800 is ready.
			chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
			while(chipid != 0x7C)
			{
				chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
				ft_delay(100);
			}
	#if defined(MSVC_PLATFORM) || defined (FT900_PLATFORM)
			printf("VC1 register ID after wake up %x\n",chipid);
	#endif
	}
	/* Configuration of LCD display */
#ifdef DISPLAY_RESOLUTION_QVGA
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 320;
	FT_DispHeight = 240;
	FT_DispHCycle =  408;
	FT_DispHOffset = 70;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 263;
	FT_DispVOffset = 13;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 2;
	FT_DispPCLK = 8;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 0;
	FT_DispCSpread = 1;
	FT_DispDither = 1;

#endif
#ifdef DISPLAY_RESOLUTION_WVGA
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 800;
	FT_DispHeight = 480;
	FT_DispHCycle =  928;
	FT_DispHOffset = 88;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 48;
	FT_DispVCycle = 525;
	FT_DispVOffset = 32;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 3;
	FT_DispPCLK = 2;
	FT_DispSwizzle = 0;
	FT_DispPCLKPol = 1;
	FT_DispCSpread = 0;
	FT_DispDither = 1;
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
	/* Values specific to HVGA LCD display */

	FT_DispWidth = 320;
	FT_DispHeight = 480;
	FT_DispHCycle =  400;
	FT_DispHOffset = 40;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 500;
	FT_DispVOffset = 10;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 5;
	FT_DispPCLK = 4;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 1;
	FT_DispCSpread = 1;
	FT_DispDither = 1;

#ifdef ME810A_HV35R
	FT_DispPCLK = 5;
#endif

#endif

#if (defined(ME800A_HV35R) || defined(ME810A_HV35R))
	/* After recognizing the type of chip, perform the trimming if necessary */
    Ft_Gpu_ClockTrimming(phost,LOW_FREQ_BOUND);
#endif

	Ft_Gpu_Hal_Wr16(phost, REG_HCYCLE, FT_DispHCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_HOFFSET, FT_DispHOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC0, FT_DispHSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC1, FT_DispHSync1);
	Ft_Gpu_Hal_Wr16(phost, REG_VCYCLE, FT_DispVCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_VOFFSET, FT_DispVOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC0, FT_DispVSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC1, FT_DispVSync1);
	Ft_Gpu_Hal_Wr8(phost, REG_SWIZZLE, FT_DispSwizzle);
	Ft_Gpu_Hal_Wr8(phost, REG_PCLK_POL, FT_DispPCLKPol);
	Ft_Gpu_Hal_Wr16(phost, REG_HSIZE, FT_DispWidth);
	Ft_Gpu_Hal_Wr16(phost, REG_VSIZE, FT_DispHeight);
	Ft_Gpu_Hal_Wr16(phost, REG_CSPREAD, FT_DispCSpread);
	Ft_Gpu_Hal_Wr16(phost, REG_DITHER, FT_DispDither);

#if (defined(FT_800_ENABLE) || defined(FT_810_ENABLE) ||defined(FT_812_ENABLE))
    /* Touch configuration - configure the resistance value to 1200 - this value is specific to customer requirement and derived by experiment */
    Ft_Gpu_Hal_Wr16(phost, REG_TOUCH_RZTHRESH,RESISTANCE_THRESHOLD);
#endif
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO_DIR,0xff);
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);


    /*It is optional to clear the screen here*/
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
    Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);


    Ft_Gpu_Hal_Wr8(phost, REG_PCLK,FT_DispPCLK);//after this display is visible on the LCD


#ifdef ENABLE_ILI9488_HVGA_PORTRAIT
	/* to cross check reset pin */
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);
	ft_delay(120);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0x7f);
	ft_delay(120);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);

	ILI9488_Bootup();

	/* Reconfigure the SPI */
#ifdef FT900_PLATFORM
	printf("after ILI9488 bootup \n");
	//spi
	// Initialize SPIM HW
	sys_enable(sys_device_spi_master);
	gpio_function(27, pad_spim_sck); /* GPIO27 to SPIM_CLK */
	gpio_function(28, pad_spim_ss0); /* GPIO28 as CS */
	gpio_function(29, pad_spim_mosi); /* GPIO29 to SPIM_MOSI */
	gpio_function(30, pad_spim_miso); /* GPIO30 to SPIM_MISO */

	gpio_write(28, 1);
	spi_init(SPIM, spi_dir_master, spi_mode_0, 4);
#endif

#endif



	/* make the spi to quad mode - addition 2 bytes for silicon */
#ifdef FT_81X_ENABLE
	/* api to set quad and numbe of dummy bytes */
#ifdef ENABLE_SPI_QUAD
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_QUAD_CHANNEL,FT_GPU_SPI_TWODUMMY);
#elif ENABLE_SPI_DUAL
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_DUAL_CHANNEL,FT_GPU_SPI_TWODUMMY);
#else
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_SINGLE_CHANNEL,FT_GPU_SPI_ONEDUMMY);
#endif

#endif

#ifdef FT900_PLATFORM
    /* Change clock frequency to 25mhz */
	spi_init(SPIM, spi_dir_master, spi_mode_0, 4);

#if (defined(ENABLE_SPI_QUAD))
    /* Initialize IO2 and IO3 pad/pin for dual and quad settings */
    gpio_function(31, pad_spim_io2);
    gpio_function(32, pad_spim_io3);
    gpio_write(31, 1);
    gpio_write(32, 1);
#endif
	/* Enable FIFO of QSPI */
	spi_option(SPIM,spi_option_fifo_size,64);
	spi_option(SPIM,spi_option_fifo,1);
	spi_option(SPIM,spi_option_fifo_receive_trigger,1);
#endif

#ifdef ENABLE_SPI_QUAD
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,4);
#endif
#elif ENABLE_SPI_DUAL	
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,2);
#endif
#else
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,1);
#endif

	
#endif


phost->ft_cmd_fifo_wp = Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE);

}


#define SAMAPP_ENABLE_APIS_SET0
#ifdef SAMAPP_ENABLE_APIS_SET0

/* Optimized implementation of sin and cos table - precision is 16 bit */
FT_PROGMEM ft_prog_uint16_t sintab[] = {
  0, 402, 804, 1206, 1607, 2009, 2410, 2811, 3211, 3611, 4011, 4409, 4807, 5205, 5601, 5997, 6392, 
  6786, 7179, 7571, 7961, 8351, 8739, 9126, 9511, 9895, 10278, 10659, 11038, 11416, 11792, 12166, 12539,
  12909, 13278, 13645, 14009, 14372, 14732, 15090, 15446, 15799, 16150, 16499, 16845, 17189, 17530, 17868,
  18204, 18537, 18867, 19194, 19519, 19840, 20159, 20474, 20787, 21096, 21402, 21705, 22004, 22301, 22594, 
  22883, 23169, 23452, 23731, 24006, 24278, 24546, 24811, 25072, 25329, 25582, 25831, 26077, 26318, 26556, 26789, 
  27019, 27244, 27466, 27683, 27896, 28105, 28309, 28510, 28706, 28897, 29085, 29268, 29446, 29621, 29790, 29955, 
  30116, 30272, 30424, 30571, 30713, 30851, 30984, 31113, 31236, 31356, 31470, 31580, 31684, 31785, 31880, 31970, 
  32056, 32137, 32213, 32284, 32350, 32412, 32468, 32520, 32567, 32609, 32646, 32678, 32705, 32727, 32744, 32757, 
  32764, 32767, 32764};




ft_int16_t qsin(ft_uint16_t a)
{
  ft_uint8_t f;
  ft_int16_t s0,s1;

  if (a & 32768)
    return -qsin(a & 32767);
  if (a & 16384)
    a = 32768 - a;
  f = a & 127;
  s0 = ft_pgm_read_word(sintab + (a >> 7));
  s1 = ft_pgm_read_word(sintab + (a >> 7) + 1);
  return (s0 + ((ft_int32_t)f * (s1 - s0) >> 7));
}

/* cos funtion */
ft_int16_t qcos(ft_uint16_t a)
{
  return (qsin(a + 16384));
}
#endif



/*****************************************************************************/
/* Example code to display few points at various offsets with various colors */




char *info[] = { 
					  "FT800 Washing Machine Demo Application",
          			  "APP to demonstrate interactive Menu,",
					  "and animation,",	 
					  "using Bitmaps",	 
					 };



float linear(float p1,float p2,ft_uint16_t t,ft_uint16_t rate)
{
	float st  = (float)t/rate;
	return p1+(st*(p2-p1));
}

ft_uint16_t smoothstep(ft_int16_t p1,ft_int16_t p2,ft_uint16_t t,ft_uint16_t rate)
{
	float dst  = (float)t/rate;
	float st = SQ(dst)*(3-2*dst);
	return p1+(st*(p2-p1));
}

ft_uint16_t acceleration(ft_uint16_t p1,ft_uint16_t p2,ft_uint16_t t,ft_uint16_t rate)
{
	float dst  = (float)t/rate;
	float st = SQ(dst);
	return p1+(st*(p2-p1));
}

float deceleration(ft_uint16_t p1,ft_uint16_t p2,ft_uint16_t t,ft_uint16_t rate)
{
	float st,dst  = (float)t/rate;
	dst = 1-dst;
	st = 1-SQ(dst);
	return p1+(st*(p2-p1));
}




static ft_uint8_t key_pressed = 0;

ft_uint8_t Read_Keys()
{
	static ft_uint8_t Read_tag=0,temp_tag=0,ret_tag=0;	
	ret_tag = NULL;		
	Read_tag = Ft_Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);						
	if(Read_tag!=NULL)											// Allow if the Key is released
	{
		if(temp_tag!=Read_tag)
		{
			temp_tag = Read_tag;	
			key_pressed = Read_tag;										// Load the Read tag to temp variable	
		}  
	}
	else
	{
		key_pressed = 0;
		if(temp_tag!=0)
		{
			ret_tag = temp_tag;
			temp_tag = 0;
		}  
	}
	return ret_tag;
}

static ft_uint8_t istouch()
{
  return !(Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_RAW_XY) & 0x8000);
}
void Use_Rgb()
{
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));	
}
void Use_Alpha()
{
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
}

void Clear_Alpha()
{
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(0));
}
void end_frame()
{
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
}



ft_void_t Play_Sound(ft_uint8_t sound,ft_uint8_t volume)
{
  Ft_Gpu_Hal_Wr8(phost,REG_VOL_SOUND,volume);
  Ft_Gpu_Hal_Wr8(phost,REG_SOUND,sound);
  Ft_Gpu_Hal_Wr8(phost,REG_PLAY,1);
}	


       
ft_void_t Info()
{
		  ft_uint16_t dloffset = 0,z=0;
		  Ft_CmdBuffer_Index = 0;
		  Ft_Gpu_CoCmd_Dlstart(phost); 
		  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
		  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2,29,OPT_CENTERX|OPT_CENTERY,"Please tap on a dot");
		  Ft_Gpu_CoCmd_Calibrate(phost,0);

		  Ft_App_Flush_Co_Buffer(phost);
		  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  
  Ft_Gpu_CoCmd_Logo(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  while(0!=Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ)); 
  dloffset = Ft_Gpu_Hal_Rd16(phost,REG_CMD_DL);
  dloffset -=4;
#ifdef FT_81X_ENABLE
  dloffset -= 2*4;//remove two more instructions in case of 81x
#endif
  
  Ft_Gpu_Hal_WrCmd32(phost,CMD_MEMCPY);
  Ft_Gpu_Hal_WrCmd32(phost,200*1024L);
  Ft_Gpu_Hal_WrCmd32(phost,RAM_DL);
  Ft_Gpu_Hal_WrCmd32(phost,dloffset);
  do
  {
    Ft_Gpu_CoCmd_Dlstart(phost);   
    Ft_Gpu_CoCmd_Append(phost,200*1024L,dloffset);
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_B(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_C(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_D(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_F(0));  
    Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());	
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(220));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
    Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());	
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
   // INFORMATION 
#ifdef DISPLAY_RESOLUTION_WVGA
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,40,30,OPT_CENTERX|OPT_CENTERY,info[0]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,80,29,OPT_CENTERX|OPT_CENTERY,info[1]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,120,29,OPT_CENTERX|OPT_CENTERY,info[2]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,160,29,OPT_CENTERX|OPT_CENTERY,info[3]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight-30,29,OPT_CENTERX|OPT_CENTERY,"Click to play");
#else
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,20,28,OPT_CENTERX|OPT_CENTERY,info[0]);
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,60,26,OPT_CENTERX|OPT_CENTERY,info[1]);
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,90,26,OPT_CENTERX|OPT_CENTERY,info[2]);  
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,120,26,OPT_CENTERX|OPT_CENTERY,info[3]);  
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight-30,26,OPT_CENTERX|OPT_CENTERY,"Click to play");

#endif
    if(key_pressed!='P')
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
    else
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(100,100,100));


	
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));   
    Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16));
    Ft_App_WrCoCmd_Buffer(phost,TAG('P'));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth/2)*16,(FT_DispHeight-60)*16));


    Ft_App_WrCoCmd_Buffer(phost,CLEAR(0,1,0));

    Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 0, 0) );
    Ft_App_WrCoCmd_Buffer(phost, STENCIL_OP(INCR,INCR) );
    Ft_App_WrCoCmd_Buffer(phost, COLOR_MASK(0,0,0,0) );//mask all the colors
    Ft_App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_L));

	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-77+5)*16));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-2+15)*16,(FT_DispHeight-67+8)*16));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-57+11)*16));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-77+5)*16));

	Ft_App_WrCoCmd_Buffer(phost, COLOR_MASK(1,1,1,1) );//enable all the colors
	Ft_App_WrCoCmd_Buffer(phost, STENCIL_FUNC(EQUAL,1,255));
	Ft_App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_L));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(FT_DispWidth*16,0));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));

	Ft_App_WrCoCmd_Buffer(phost, STENCIL_OP(KEEP,KEEP));
	Ft_App_WrCoCmd_Buffer(phost, STENCIL_FUNC(ALWAYS,0,255));
	Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));

	Ft_App_WrCoCmd_Buffer(phost, BEGIN(LINE_STRIP));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-77+5)*16));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-2+15)*16,(FT_DispHeight-67+8)*16));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-57+11)*16));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-77+5)*16));
	
    Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
    Ft_Gpu_CoCmd_Swap(phost);
    Ft_App_Flush_Co_Buffer(phost);
    Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
	
  }while(Read_Keys()!='P');

		  Ft_Gpu_CoCmd_Dlstart(phost);   
		  Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(255,255,255));
		  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));   
		  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
		  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2+40,29,OPT_CENTERX|OPT_CENTERY,"Data Loading");
#endif
#if defined(DISPLAY_RESOLUTION_WQVGA)
		  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2-80,FT_DispHeight/2,29,OPT_CENTERX|OPT_CENTERY,"Data Loading");
#endif
#if defined(DISPLAY_RESOLUTION_WVGA)
		  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2-120,FT_DispHeight/2,29,OPT_CENTERX|OPT_CENTERY,"Data Loading");
#endif
		  Ft_Gpu_CoCmd_Spinner(phost,FT_DispWidth/2+40,FT_DispHeight/2,1,0);
		  Ft_App_Flush_Co_Buffer(phost);
		  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
		  Play_Sound(0x50,255);

#if defined(FT_801_ENABLE) || defined(FT_811_ENABLE) || defined(FT_813_ENABLE)
		  Ft_Gpu_Hal_Wr8(phost,REG_CTOUCH_EXTENDED, CTOUCH_MODE_EXTENDED);
#endif
}

static struct {
  signed short dragprev;
  int vel;      // velocity
  long base;    // screen x coordinate, in 1/16ths pixel
  long limit;
} scroller;

static void scroller_init(ft_uint32_t limit)
{
  scroller.dragprev = -32768;
  scroller.vel = 0;      // velocity
  scroller.base = 0;     // screen x coordinate, in 1/16ths pixel
  scroller.limit = limit;
}
static void scroller_run()
{
  signed short sy = Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);
  
  if ((sy != -32768) & (scroller.dragprev != -32768)) {
    scroller.vel = (scroller.dragprev - sy) << 4;
  } else {
    int change = max(1, abs(scroller.vel) >> 5);
    if (scroller.vel < 0)
      scroller.vel += change;
    if (scroller.vel > 0)
      scroller.vel -= change;
  }
  scroller.dragprev = sy;
  
  scroller.base += scroller.vel;
  
  scroller.base = max(0, min(scroller.base, scroller.limit));
}

 typedef struct logoim
{
  ft_prog_char8_t name[14];
  ft_uint16_t image_height;
  ft_uint8_t image_format;
  ft_uint8_t filter;
  ft_uint16_t sizex;
  ft_uint16_t sizey;
  ft_uint16_t linestride;
  ft_uint16_t gram_address;
}t_imageprp;

FT_PROGMEM t_imageprp Main_Icons[10] = 
{
                                        
                                        	{"shirt.bin",		40,    ARGB4,  NEAREST,    40,    40 ,    80,   0L      }, 
                                		{"Temp.bin",		40,    ARGB4,  NEAREST,    40,    40 ,    80,   3L      },  
                                		{"SpinW.bin",           40,    ARGB4,  NEAREST,    40,    40 ,    80,   7L      },  
                                		{"Butfly.bin",	        20,    ARGB4,  NEAREST,    20,    20 ,    40,   11L     },  
                                	//	{"DelSt.bin",		30,    ARGB4,  BILINEAR,    30,    30 ,    60,  12L     },  
                                		{"Lock.bin",		30,    ARGB4,  NEAREST,    60,    60 ,    60,   12L     },  
                                		{"UnLock.bin",		30,    ARGB4,  NEAREST,    30,    60 ,    60,   14L     },  
                                		{"Set.bin",		30,    ARGB4,  NEAREST,    30,    30 ,    60,   16L     },  
                                		{"Start.bin",		30,    ARGB4,  NEAREST,    30,    30 ,    60,   18L     },  
                                		{"home.bin",		40,    ARGB4,  NEAREST,    40,    40 ,    80,   20L     },//4kb
                                		{"Bkgd.bin",            136,   L8,     NEAREST,    480,   272 ,   240,  24L     },

},

Wash_Window[7] = 
{
	                                        {"Wash.bin",		55,	ARGB4,	NEAREST,   50,	  50 ,	  100,	57L	}, // size 22KB
                                        	{"Rinse.bin",		54,	ARGB4,	NEAREST,   54,	  54 ,	  108,	79L	}, // size 24KB
                                        	{"Spin.bin",		54,	ARGB4,	NEAREST,   45,	  45 ,	  90,	103L	}, // size 30KB
                                        	{"WBar.bin",		52,	RGB565,	NEAREST,   400,	  400 ,	  800,	133L	}, //size 42KB
                                        	{"bub.bin",		30,	ARGB4,	NEAREST,   30,	  30 ,	  60,	175L	},//2kb
                                        	{"home.bin",		40,	ARGB4,	NEAREST,   40,	  40 ,	  80,	177L	},//4kb
                                        	{"Bkgd.bin",		136,    L8,	NEAREST,   480,	  272 ,	  240,	181L	},   //33kb
};

	ft_uint8_t Stflag,OptionsSet,MainFlag,Tagval,Pf=0;
	ft_uint16_t BrightNess,SoundLev=255,TimeSet,IterCount,Px;
	static ft_uint16_t ButX=330;

void load_inflate_image(ft_uint8_t address,char* name)
{
    ft_uint8_t ibuff[512];    
    Reader imfile;
    
    byte file = imfile.openfile(name);    
    ft_int32_t fsize = imfile.size; 
    
    Ft_Gpu_Hal_WrCmd32(phost,CMD_INFLATE);

    Ft_Gpu_Hal_WrCmd32(phost,address*1024L);
    while(fsize > 0)
    {
        
        uint16_t n = fsize > 512L ? 512L :fsize;
        imfile.readsector(ibuff);     
        Ft_Gpu_Hal_WrCmdBuf(phost,ibuff,n);				/* copy data continuously into command memory */
        fsize -= n;
    }
}




void Logo_Intial_setup(FT_PROGMEM struct logoim sptr[],ft_uint8_t num )
{
   char buffer1[10]; 
   for(ft_uint8_t z=0;z<num;z++)
   {
       ft_uint16_t pc = ft_pgm_read_word(&sptr[z].gram_address);
 
       ft_prog_char8_t *pc1 = sptr[z].name;
                      while (pgm_read_byte(pc1)) {
                      strcpy_P(buffer1, sptr[z].name);
                      pc1 += (strlen_P(pc1) + 1);
                      }
       load_inflate_image(pc,buffer1);
   }     
   Ft_Gpu_CoCmd_Dlstart(phost);        // start
   Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));

   for(ft_uint8_t z=0;z<num;z++)
   {
	
           Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(z));	
           
           ft_uint16_t pc = ft_pgm_read_word(&sptr[z].gram_address);     
         
             
           Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(pc*1024L));    
         
           ft_uint8_t pc2 = ft_pgm_read_byte(&sptr[z].image_format);
           ft_uint16_t pc3 = ft_pgm_read_word(&sptr[z].linestride);                      
           ft_uint16_t pc4 = ft_pgm_read_word(&sptr[z].image_height);

     
   
           Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(pc2,pc3,pc4));      
          
           pc2 = ft_pgm_read_byte(&sptr[z].filter);
           pc3 =ft_pgm_read_word(&sptr[z].sizex);                      
           pc4 = ft_pgm_read_word(&sptr[z].sizey);
         
     
           Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(pc2,BORDER,BORDER,pc3,pc4)); 
           
      
   }
   
   Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
   Ft_Gpu_CoCmd_Swap(phost);
   Ft_App_Flush_Co_Buffer(phost);
   Ft_Gpu_Hal_WaitCmdfifo_empty(phost);    

}

static void rotate_around(ft_int16_t x, ft_int16_t y, ft_int16_t a)
{
  Ft_Gpu_CoCmd_LoadIdentity(phost);
  Ft_Gpu_CoCmd_Translate(phost,F16(x),F16(y));
  Ft_Gpu_CoCmd_Rotate(phost,a);
  Ft_Gpu_CoCmd_Translate(phost,F16(-x),F16(-y));
  Ft_Gpu_CoCmd_SetMatrix(phost);
}


FT_PROGMEM ft_prog_char8_t Array_Cycle_Name[9][12] = {"\0","Eco Cold", "Normal", "Heavy","Perm",  "Wool", "Bedding", "ActiveWear"};
FT_PROGMEM ft_prog_char8_t Array_Cycle_Options[3][12] = {"Soil Level", "Temperature", "Spin Speed"};
FT_PROGMEM ft_prog_char8_t Array_Menu_Options[3][12] = { "Child Lock", "Settings", "Start"};



FT_PROGMEM ft_prog_char8_t BrightLevel[6][10] = {"\0","Level1", "Level2", "Level3", "Level4", "Level5"};

FT_PROGMEM ft_prog_char8_t SoundLevel[6][10] = {"\0","Mute", "Level1", "Level2", "Level3", "Level4"};

typedef struct Item_Prop
{
		ft_prog_char8_t SoilLev[8];
		ft_prog_char8_t Temp[8];
		ft_prog_char8_t SpinSpeed[8];	
}Item_Properties;

Item_Properties Item_Property[] FT_PROGMEM= 
{
	{"Light", "Cold", "Low"},//Eco cold
	{"Normal", "Warm", "Medium"}, //Normal
	{"Heavy", "Hot", "High"}, // HEavy Duty
	{"Light", "Warm", "Medium"},//Perm Press
	{"Normal", "Hot", "Medium"},//Active Wear
	{"Heavy", "Warm", "High"},//Bedding
	{"Heavy", "Cold", "Low"},//Wool
};

typedef struct Bub
{
	ft_int16_t xOffset;
	ft_int16_t yOffset;
	ft_char8_t xDiff;
	ft_char8_t yDiff;

}Bubbles;

static ft_uint8_t ProcFinish;
ft_uint16_t WashWindow()
{
		ft_uint16_t ItCt=0,BitmapTr=0,BubY=0,i=0,th=0,PwY=0,PwY1=0,PwY2=0;
		ft_int16_t RotX=0,RotY=0;
		
		//SYSTEMTIME st;
		static ft_uint8_t ProcessFlag;
		ft_uint8_t Read_tag=0,Sdflag=0,Sd1=0,Sd2=0,SoundFlag =0;

	Bubbles W_Bubble[40];
				
	Logo_Intial_setup(Wash_Window,7);

	if(MainFlag == 1)
	{
		TimeSet = 36;
		Px=44;
		//MainFlag =0;
	}
	BubY = 110;

	for(i=0;i<40;i++)
	{
		W_Bubble[i].xOffset = ft_random(FT_DispWidth);
		W_Bubble[i].yOffset = ft_random(FT_DispHeight);

		W_Bubble[i].xDiff = ft_random(16) ;
		W_Bubble[i].yDiff = ft_random(8) ;
	}

	
		
		 do
		{
			 
				Ft_Gpu_CoCmd_Dlstart(phost);        // start
				Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,255));
				Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));

		Read_tag = Read_Keys();

		/* Draw all the background bitmaps - balls with various resolutions */
		if(ProcessFlag == 2)
		{
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255 ,255 ,255)); // brown outside circle
                        if(Pf ==0)
			{
        			for(i = 0;i<25;i++)
        			{
          			        Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(W_Bubble[i].xOffset,W_Bubble[i].yOffset,4,0));
        				W_Bubble[i].xOffset += W_Bubble[i].xDiff;
        				W_Bubble[i].yOffset += W_Bubble[i].yDiff;
        
        				if(W_Bubble[i].xOffset < -80)
        				{
        					W_Bubble[i].xOffset = FT_DispWidth + 80;
        				}
        				else if(W_Bubble[i].xOffset > FT_DispWidth + 80)
        				{
        					W_Bubble[i].xOffset = -80;
        				}
        				if(W_Bubble[i].yOffset < -80)
        				{
        					W_Bubble[i].yOffset = FT_DispHeight + 80;
        				}
        				else if(W_Bubble[i].yOffset > FT_DispHeight + 80)
        				{
        					W_Bubble[i].yOffset = -80;
        				}
      			      }
                        }
			W_Bubble[ft_random(40)].xDiff = ft_random(16) - 6;
			W_Bubble[ft_random(40)].yDiff = ft_random(8) - 4;
		}
		// Progress bar Image
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));			
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));

		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(180));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128));			
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128));	
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,0,6,0));// backgnd tag -13

		Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());

		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(40,130,3,0));

                // Pause Button

		Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost, TAG('P'));
 		if(ItCt % 100 >0 && ItCt % 100 <50 && Pf == 1 && ProcessFlag<5)
			Ft_Gpu_CoCmd_Text(phost,205,190,31,0,"I I"); 				
		else if(Pf == 0 && ProcessFlag<5)
			Ft_Gpu_CoCmd_Text(phost,205,190,31,0,"I I"); 				
		Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(0));

		if(Read_tag == 'P') 
		{
			if(Pf == 1)Pf = 0;
			else if(Pf ==0)Pf =1;
		}

		//Progress bar cover up
		if(Px <435)
                {
                        //Px = 435;
        		Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));		
        		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(129,178,218));
        		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(5*16));
        		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Px*16,134*16));
        		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(435*16,178*16));
                }

		Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));		
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(100));
		Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(45*16));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(225*16,85*16));

		// Wash image
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));

                if(Pf == 0)
		{
        		if(TimeSet > 30)
        		{
        			ProcessFlag = 1;
        			Ft_Gpu_CoCmd_Text(phost,40,190,23,0,"Pre Wash"); 
					ProcFinish =1;
        		}
        		else if(TimeSet > 20 && TimeSet <= 30)
        		{
        			Ft_Gpu_Hal_Wr8(phost,REG_VOL_PB,0);
        			if(Sdflag == 0)
        			{
        				 Play_Sound(0x56,SoundLev);
        				 Sdflag = 1;
        			}
        			//Ft_Gpu_Hal_Wr8(phost,REG_PLAYBACK_PLAY,0);
        			ProcessFlag = 2;
        			Ft_Gpu_CoCmd_Text(phost,40,190,23,0,"Washing"); 
					ProcFinish =1;
        		}
        		else if(TimeSet <= 20 && TimeSet >10) 
        		{
        			if(Sd1 == 0)
        			{
        				 Play_Sound(0x56,SoundLev);
        				 Sd1 = 1;
        			}
        			ProcessFlag = 3;
        			Ft_Gpu_CoCmd_Text(phost,40,190,23,0,"Rinsing"); 
					ProcFinish =1;
        		}
        		else if(TimeSet <= 10 && TimeSet >0) 
        		{
        			if(Sd2 == 0)
        			{
        				 Play_Sound(0x56,SoundLev);
        				 Sd2 = 1;
        			}
        			ProcessFlag = 4;
        			Ft_Gpu_CoCmd_Text(phost,40,190,23,0,"Spinning"); 
					ProcFinish =1;
        		}
        		else if(TimeSet == 0) 
        		{
        			ProcessFlag = 5;
        			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));
        			if(ItCt %20 >0 && ItCt %20 < 10)
        			Ft_Gpu_CoCmd_Text(phost,197,73,23,0,"DONE"); 
        			MainFlag = 0;
					ProcFinish =2;
					ButX = 330;
					Stflag = 0;
        		}
        		{
        			if(ItCt%25 == 0)
        				BitmapTr = 0;
        			else	if(ItCt%40 == 0)
        				BitmapTr = 1;
        			else	if(ItCt%60 == 0)
        				BitmapTr = 2;
        			else	if(ItCt%90 == 0)
        				BitmapTr = 3;
        		}
                }
		// Process Images
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));			
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		// Pre Wash Image
		if(ProcessFlag == 1)// Images loading during pre wash stage
		{
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(200,PwY,1,0));
			if(PwY == 61)
			{
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(200,PwY1,0,0));
			}
			if(PwY1 == 61)
			{
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(200,PwY2,2,0));
			}
		}
		else	if(ProcessFlag < 5)
		{
			if(ProcessFlag == 2 || ProcessFlag == 3)
			{
				RotX = 27;RotY = 27;
				Ft_App_WrCoCmd_Buffer(phost,CMD_LOADIDENTITY);
                                if(Pf == 0)
				rotate_around(RotX,RotY,th);
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(200,60,ProcessFlag-2,BitmapTr));						
				Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());	
			}

			else if(ProcessFlag == 4)
			{
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(200,60,ProcessFlag-2,BitmapTr));					
			}
		}

		// Home Button
		Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost, TAG('H'));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(430,10,5,0));
		Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(0));

		// TIme remainig
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));		
		Ft_Gpu_CoCmd_Number(phost, 40,  60, 31, OPT_CENTERX|OPT_CENTERY, 0		); 
		Ft_Gpu_CoCmd_Number(phost, 60,  60, 31, OPT_CENTERX|OPT_CENTERY, 0		); 
		Ft_Gpu_CoCmd_Text  (phost, 75,  67, 21, OPT_CENTERX|OPT_CENTERY, "H"		); 
		Ft_Gpu_CoCmd_Text  (phost, 85,  57, 31, OPT_CENTERX|OPT_CENTERY, ":"		); 
		if(TimeSet < 10)
		{
			Ft_Gpu_CoCmd_Number(phost,105,60,31,OPT_CENTERX|OPT_CENTERY,0); 
			Ft_Gpu_CoCmd_Number(phost,128,60,31,OPT_CENTERX|OPT_CENTERY,TimeSet); 
		}
		else
			Ft_Gpu_CoCmd_Number(phost, 115, 60, 31, OPT_CENTERX|OPT_CENTERY, TimeSet	); 
		Ft_Gpu_CoCmd_Text  (phost, 150, 67, 21, OPT_CENTERX|OPT_CENTERY, "Min"		); 				
			
		//GetLocalTime(&st);
		// Est Finish time
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		Ft_Gpu_CoCmd_Text(phost,300,190,23,0,"Est. Finish Time"); 

		
                Ft_Gpu_CoCmd_Number(phost,315,212,25,0,0); 
                Ft_Gpu_CoCmd_Number(phost,335,212,25,0,0); 
		Ft_Gpu_CoCmd_Text(phost,355,210,25,0,":"); 
                if(TimeSet < 10)
		{
			Ft_Gpu_CoCmd_Number(phost, 370    , 212, 25, 0, 0	); 
			Ft_Gpu_CoCmd_Number(phost, 390    , 212, 25, 0, TimeSet	); 
		}
		else
                    Ft_Gpu_CoCmd_Number(phost, 370    , 212, 25, 0, TimeSet	); 
                

                if(TimeSet%10!=0)
                {
                  if(TimeSet % 2 == 0)
                  {
			if(SoundFlag == 0)
			{
				Play_Sound(0x50,SoundLev);
				SoundFlag = 1;
			}
                  }
		  else
		        SoundFlag =0;
                }
		Ft_App_WrCoCmd_Buffer(phost,END());			
		Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
		Ft_Gpu_CoCmd_Swap(phost);
		Ft_App_Flush_Co_Buffer(phost);                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                              
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);     

						
		ItCt+=1;
		if(ItCt%50==0 && Pf == 0)	TimeSet--;
		if(TimeSet	>60)	TimeSet =0;
		// Process rectangle cover up co ordinates
		if(IterCount %50==0 && Pf == 0)
		  Px+=11;
		//Butterfly Line
		if(IterCount%15 == 0  && Pf==0 )ButX+=1;			

		if(ProcessFlag == 3)
		{
			if((ItCt%100 > 0) &&(ItCt%100 <50))
				th-=1000;
			if((ItCt%100 >= 50) &&(ItCt%100 <100))			
				th+= 1000;
		}
	
		if(PwY <61)PwY+=1;
		if(PwY1 <61 && PwY == 61)PwY1+=1;
		if(PwY2 <61 && PwY1 == 61 )PwY2+=1;
			IterCount+=1;

		}while(Read_tag !='H');
	OptionsSet = 1;
	return  Read_tag;			
}


static ft_uint8_t BrightKey,VolKey;
ft_uint8_t Settings()
{
	ft_uint8_t Read_tag=0,i=0,SoundFlag=0;
	ft_uint16_t Rx1 =0,Ry1=0,Rx2=0,Ry2=0;
        char buffer[10];
	if(BrightKey == 0)BrightKey = 5;
		if(VolKey == 0)VolKey = 10;
	do{
  
		Read_tag = Read_Keys();
		Ft_Gpu_CoCmd_Dlstart(phost);   
			
		Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,255));
		Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));

		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(180));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128));			
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128));	
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,0,9,0));// backgnd tag -8

		Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());

		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,200,255));

		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost,TAG('V'));
		Ft_Gpu_CoCmd_Text(phost,60,80,23,OPT_CENTERX|OPT_CENTERY,"Display");

		Ft_App_WrCoCmd_Buffer(phost,TAG('B'));
		Ft_Gpu_CoCmd_Text(phost,60,145,23,OPT_CENTERX|OPT_CENTERY,"Sound");
					
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));	

		Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
		
		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(4*16));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(100));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(115*16, 25*16));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(415*16, 240*16));

		//Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(4*16));
			
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		// Brightness
		Rx1 = 120;Ry1 = 70;Rx2 = 170;Ry2 = 100;
		for(i =1;i<6;i++)
		{
			if(i==1)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(177,176,180));
					
			if(i==2)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(122,154,171));

			if(i==3)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(70,149,186));

			if(i==4)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(28,140,202));

			if(i==5)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(135,183,71));
				
			if(i == BrightKey)
			{
				Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,BrightNess);
				Ft_App_WrCoCmd_Buffer(phost,TAG(i));		
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Rx1-5)*16, (Ry1-5)*16));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Rx2+5)*16, (Ry2+5)*16));
			}
			else
			{
				Ft_App_WrCoCmd_Buffer(phost,TAG(i));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Rx1*16, Ry1*16));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Rx2*16, Ry2*16));
			}
			Rx1+= 60;
			Rx2+= 60;
		}
		Rx1 = 143;Ry1 = 73;
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
		
		for(i =1;i<6;i++)
		{
			Ft_App_WrCoCmd_Buffer(phost,TAG(i));
			strcpy_P(buffer, BrightLevel[i]);   
			Ft_Gpu_CoCmd_Text(phost,Rx1,Ry1+10,18,OPT_CENTERX|OPT_CENTERY,buffer);
			Rx1+= 60;
			Rx2+= 60;
		}

			
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

		Ft_App_WrCoCmd_Buffer(phost,END());

		// Volume
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(4*16));

		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		Rx1 = 120;Ry1 = 130;Rx2 = 170;Ry2 = 160;
		for(i =1;i<6;i++)
		{
	                if(i==1)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(177,176,180));

			if(i==2)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(122,154,171));

			if(i==3)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(70,149,186));

			if(i==4)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(28,140,202));

			if(i==5)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(135,183,71));

			if(i == VolKey-5)
			{
				if(SoundFlag == 0)
				{
					Play_Sound(0x56,SoundLev);
					SoundFlag =1;
				}
				Ft_App_WrCoCmd_Buffer(phost,TAG(i+5));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Rx1-5)*16, (Ry1-5)*16));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((Rx2+5)*16, (Ry2+5)*16));
			}
			else
			{
				Ft_App_WrCoCmd_Buffer(phost,TAG(i+5));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Rx1*16, Ry1*16));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Rx2*16, Ry2*16));
			}
			Rx1+= 60;
			Rx2+= 60;
		}
		Rx1 = 143;Ry1 = 133;
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
		
		for(i =1;i<6;i++)
		{
                        Ft_App_WrCoCmd_Buffer(phost,TAG(i+5));
		        strcpy_P(buffer, SoundLevel[i]);   
			Ft_Gpu_CoCmd_Text(phost,Rx1,Ry1+10,18,OPT_CENTERX|OPT_CENTERY,buffer);
			Rx1+= 60;
			Rx2+= 60;
		}

		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
		Ft_App_WrCoCmd_Buffer(phost,END());

		if((Read_tag >0) && (Read_tag <6) )
		{
				BrightKey =Read_tag;				
		}
		if((Read_tag >5) && (Read_tag <11) )
		{
				VolKey =Read_tag;
				SoundFlag = 0;
		}
		if( Read_tag >0 && Read_tag < 6 )	
			BrightNess = Read_tag*128/5;
		if( Read_tag >5 && Read_tag < 11 )
			SoundLev = (Read_tag-6)*63;

		// Home Button
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
		Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost, TAG('H'));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(430,10,8,0));
		Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(0));

		Ft_App_WrCoCmd_Buffer(phost,END());
		
		Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
		Ft_Gpu_CoCmd_Swap(phost);
		Ft_App_Flush_Co_Buffer(phost);
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);      

	}while(Read_tag != 'H');
return Read_tag;
}

static  ft_uint8_t MenuFlag;
ft_uint8_t MainWindow()
{
        char buffer[20];
	ft_uint16_t Ty=0,By=0,Ly=0,th=0,Addth=0;
	ft_uint8_t PrevTag=0,Read_Tag=0,temp=0,dg_count=0,LockFlag=0,PlayFlag=0,Opt=0,i=0;
	ft_int16_t sy = 0,	drag = 0,ret=0;
	static ft_uint8_t j,k;
	scroller_init((FT_DispHeight*2)*16);
        char ItemProp[20],ItemSp[10],ItemSp1[10],ItemSp2[10],ItemSp3[10];  


	do
	{

		Ft_Gpu_Hal_Wr8(phost,REG_VOL_SOUND,0);
		Ft_Gpu_Hal_Wr8(phost,REG_VOL_PB,0);
			
		
		sy =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);
		Read_Tag = Read_Keys();
			
		if(sy != -32768 )	// touch
		{
			dg_count++;     	
			temp = key_pressed;      
		}
		/*Move into the particular frame based on dragdt now 30pixels*/
		if(sy==-32768)
		{
			dg_count = 0;			 
			drag = scroller.base>>4; 	
		}
		/*if tag in but still pendown take a scroller basevalue*/
		else if(dg_count>15)
		{
			  Read_Tag = 0;
			  temp = key_pressed;
			  drag = scroller.base>>4;
		}
		if(drag>= 135) 
		{
			drag = 135;
		}
		if(temp == 101/* ||((temp > 0) &&(temp < 8)))*/ && drag <= 135)
		{		
			scroller_run();
		}
		Ft_Gpu_CoCmd_Dlstart(phost);        // start
		Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,255));
		Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));

	        // BackGround Image
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(180));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128));			
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128));	
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,0,9,0));// backgnd tag -8
		Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());

		// Draw rectanglw with zero alpha value to have scrolling option on left pane
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(0));
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost,TAG(101));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0*16, 0*16));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(120*16, 2720*16));
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
		Ft_App_WrCoCmd_Buffer(phost,END());
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
		Ty = 30;

		Ty = Ty-drag;
		// Options on the left pane
		for(i =1; i<8;i++)
		{
			if(i == j)
			{
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
			}
			else
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,200,255));
    			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
			Ft_App_WrCoCmd_Buffer(phost,TAG(i));
                        strcpy_P(buffer, Array_Cycle_Name[i]);                           
			Ft_Gpu_CoCmd_Text(phost,60,Ty,23,OPT_CENTERX|OPT_CENTERY, buffer);					
			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));				
			Ty = Ty+ 50;
		}
		
		// Draw rectangle in the centre to show the options
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
		
		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(4*16));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(100));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(140*16, 25*16));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(320*16, 240*16));
		
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
		Ft_Gpu_CoCmd_Text(phost,220,32,23,OPT_CENTERX|OPT_CENTERY, "Options"); 

		// Display icons in vertical order
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));			
		By = 60;
		for(i=0;	i<3;	i++)
		{
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
			if(i<2)
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(150,By,i,0)); 
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
                        strcpy_P(buffer, Array_Cycle_Options[i]);   
			Ft_Gpu_CoCmd_Text(phost,240,By+10,18,OPT_CENTERX|OPT_CENTERY, buffer); 
			By+=60;
		}

		// Lines draw after each icon
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(100));					
		Ly = 55;
		for(i=0;	i<3;	i++)
		{
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(140*16, Ly*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(320*16, Ly*16));
			Ly += 60;
		}
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));		
		By = 90;
		if(key_pressed > 0 && key_pressed < 9)
		{
			if(ProcFinish == 0 || ProcFinish ==2 || TimeSet ==0)
			{
				Tagval =key_pressed;
				OptionsSet =1;
				temp =0;
				j =key_pressed;
			}
		}
		if(Tagval !=0)
		{
			for(i=0;	i<3;	i++)
			{
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,255));
				if(i==0)
                                {
                                        strcpy_P(ItemProp, Item_Property[Tagval-1].SoilLev);
					Ft_Gpu_CoCmd_Text(phost,240,By,29,OPT_CENTERX|OPT_CENTERY, ItemProp); 
                                }
				if(i==1)
                                {
                                        strcpy_P(ItemProp, Item_Property[Tagval-1].Temp);
				        Ft_Gpu_CoCmd_Text(phost,240,By,29,OPT_CENTERX|OPT_CENTERY, ItemProp); 
                                }
				if(i==2)
                                {
                                        strcpy_P(ItemSp, Item_Property[Tagval-1].SpinSpeed);
                                       	Ft_Gpu_CoCmd_Text(phost,240,By,29,OPT_CENTERX|OPT_CENTERY, ItemSp); 
                                }
				By+=60;
			}
		}


		// for soil icon
                strcpy_P(ItemProp, Item_Property[Tagval-1].SoilLev);
                strcpy(ItemSp1, "Light");
                strcpy(ItemSp2, "Normal");
                strcpy(ItemSp3, "Heavy");
                ret = strcmp(ItemProp, ItemSp1);
                if(ret ==0 )
                {
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(200,200,200));
                        k=1;
                }
		ret = strcmp(ItemProp, ItemSp2);
                if(ret ==0 )
                {
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(150,150,150));
                        k =2;
                }
			
		ret = strcmp(ItemProp, ItemSp3);
                if(ret ==0 )
                {
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(80,80,80));//orange
                        k = 3;
                }
		if((k != 1) && (k !=2) && (k!=3))
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		 Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
		 Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(150,60,0,0)); 


		// for temp icon - draw point and line inside icon
                strcpy_P(ItemProp, Item_Property[Tagval-1].Temp);
                strcpy(ItemSp1, "Cold");
                strcpy(ItemSp2, "Warm");
                strcpy(ItemSp3, "Hot");
                ret = strcmp(ItemProp, ItemSp1);
                if(ret ==0 )
                      Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,255));//blue
                      
                ret = strcmp(ItemProp, ItemSp2);
                if(ret ==0 )
                      Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,140,0));//blue
                      
                ret = strcmp(ItemProp, ItemSp3);
                if(ret ==0 )
                      Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));//red
		
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));		
		Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(5*16));	
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(170*16, 150*16));
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(170*16, 145*16));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(170*16, 135*16));
                strcpy(ItemSp1, "Low");
                strcpy(ItemSp2, "Medium");
                strcpy(ItemSp3, "High");
                ret = strcmp(ItemSp, ItemSp1);
                if(ret ==0 )
                      Addth = 500;   
                          
                ret = strcmp(ItemSp, ItemSp2);
                if(ret ==0 )
                      Addth = 1000;   
                          
                ret = strcmp(ItemSp, ItemSp3);
                if(ret ==0 )
                      Addth = 1500;   
                      
                       
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,255));
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
		Ft_App_WrCoCmd_Buffer(phost,CMD_LOADIDENTITY);
		rotate_around(20,20,th);			
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(150,180,2,0)); 
		Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());	
			
                if(Stflag == 0)
				TimeSet = 0;
		if(TimeSet > 36)TimeSet = 0;
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,200,255));
		Ft_Gpu_CoCmd_Text(phost,400,30,18,OPT_CENTERX|OPT_CENTERY,"Estimated Time"); 
		Ft_Gpu_CoCmd_Number(phost,355,60,31,OPT_CENTERX|OPT_CENTERY,0); 
		Ft_Gpu_CoCmd_Text(phost,372,67,21,OPT_CENTERX|OPT_CENTERY,"H"); 
		if(TimeSet < 10)
		{
			Ft_Gpu_CoCmd_Number(phost,395,60,31,OPT_CENTERX|OPT_CENTERY,0); 
			Ft_Gpu_CoCmd_Number(phost,415,60,31,OPT_CENTERX|OPT_CENTERY,TimeSet); 
		}
		else
			Ft_Gpu_CoCmd_Number(phost,405,60,31,OPT_CENTERX|OPT_CENTERY,TimeSet); 
		Ft_Gpu_CoCmd_Text(phost,442,67,21,OPT_CENTERX|OPT_CENTERY,"Min"); 


		Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
		Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(92,148,212));					
		//Blue line below butterfly
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(340*16, 90*16));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(460*16, 90*16));
		//Orange line below butterfly
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,140,0));	
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(340*16, 90*16));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(ButX*16, 90*16));
		//Butterfly			
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		if(ButX > 460)ButX =460;
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(ButX,75 ,3,0)); 

		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));

		//Lock & Unlock Buton
		if(Read_Tag == 'U')LockFlag = 0;
			
		//if(LockFlag =='U' || LockFlag == 0)
		{
			Ft_App_WrCoCmd_Buffer(phost,TAG('L'));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(340,105 ,4,0)); 
		}
			
		if(Read_Tag == 'L')LockFlag = 1;

		

		// Settings Button
		Ft_App_WrCoCmd_Buffer(phost,TAG('S'));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(340,145 ,6,0)); 

		// Start Button
		Ft_App_WrCoCmd_Buffer(phost,TAG('P'));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(340,185 ,7,0)); 

		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

		if(Read_Tag =='S')
                {
                     Settings();
                }
		if(BrightNess ==0)BrightNess =100;
		Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,BrightNess);// start

                if(TimeSet == 0) ProcFinish=2;
		if((Read_Tag == 'P'))
		{
			PlayFlag = 1;
			Opt=0;
                        if(ProcFinish == 2)
			{
				TimeSet = 36;
				ButX =330;
				Px = 44;
			}
		}
		if(PlayFlag ==1)
		{
			if(	OptionsSet == 1)
			{
				Stflag = 1;
				OptionsSet = 2;
			}
			else if(OptionsSet != 1 && Opt !=1)
			{
				if((IterCount % 100 > 0)&&( IterCount % 100 < 100))
				{
					Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));
					Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
					Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(110*16,115*16));
					Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(375*16,140*16));
					Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
					Ft_Gpu_CoCmd_Text(phost,120,125,27,OPT_CENTERY,"Please Select the Washing Mode");
					if(IterCount % 100 == 99)
					{
						Opt=1;
						PlayFlag=0;
					}
				}
			}
			
		}
				
		if(temp >= 50)temp =0;
		Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
		By = 115;
		for(i=0;i<3;i++)
		{
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,200,255));
                        strcpy_P(buffer, Array_Menu_Options[i]);                 
			Ft_Gpu_CoCmd_Text(phost,380,By,18,0,buffer); 
			By+=40;
		}
		if(LockFlag ==1)
		{
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(180));
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,FT_DispHeight*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
			Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());	

			Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128) );
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128) );
			Ft_App_WrCoCmd_Buffer(phost,TAG('U'));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(220,105 ,4,0));
			Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());	
		}
			
		if(MainFlag == 0) Px = 44;
		Ft_App_WrCoCmd_Buffer(phost,END());			
		Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
		Ft_Gpu_CoCmd_Swap(phost);
		Ft_App_Flush_Co_Buffer(phost);
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);       
		th+=Addth;
		if(IterCount%15 == 0 && Stflag == 1  && Pf ==0)
			ButX+=1;			
		if(IterCount%50 == 0  && Pf ==0 && Stflag == 1)TimeSet--;
		if(IterCount %50 == 0 && Pf ==0	 )	Px+=11;
						
		if(temp >=1 )temp++;
		IterCount+=1;
	}while(OptionsSet !=2 );//(Read_Tag!= 'P');
	MainFlag ++;
	return Read_Tag;
}
#ifdef MSVC_PLATFORM
/* Main entry point */
ft_int32_t main(ft_int32_t argc,ft_char8_t *argv[])
#endif
#ifdef ARDUINO_PLATFORM
ft_void_t setup()
#endif
{
      
      
	ft_uint8_t Val,chipid;
        static ft_uint8_t sd_present =0;

	Ft_Gpu_HalInit_t halinit;
	
	halinit.TotalChannelNum = 1;

              
	Ft_Gpu_Hal_Init(&halinit);
	host.hal_config.channel_no = 0;
	host.hal_config.pdn_pin_no = FT800_PD_N;
	host.hal_config.spi_cs_pin_no = FT800_SEL_PIN;
#ifdef MSVC_PLATFORM_SPI
	host.hal_config.spi_clockrate_khz = 12000; //in KHz
#endif
#ifdef ARDUINO_PLATFORM_SPI
	host.hal_config.spi_clockrate_khz = 4000; //in KHz
#endif
        Ft_Gpu_Hal_Open(&host);
            
	phost = &host;

      Ft_BootupConfig();

 


        pinMode(FT_SDCARD_CS, OUTPUT);
        digitalWrite(FT_SDCARD_CS, HIGH);
        delay(100);
        sd_present =  SD.begin(FT_SDCARD_CS);
      
      
        SPI.setClockDivider(SPI_CLOCK_DIV2);
        SPI.setBitOrder(MSBFIRST);
        SPI.setDataMode(SPI_MODE0);
	
	
	if(!sd_present)
        {
  	    Ft_Gpu_CoCmd_Dlstart(phost);    
            Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
            Ft_App_WrCoCmd_Buffer(phost,SCISSOR_XY(FT_DispWidth/12,FT_DispHeight/9));
            Ft_App_WrCoCmd_Buffer(phost,SCISSOR_SIZE(FT_DispWidth-2*(FT_DispWidth/12),FT_DispHeight-2*(FT_DispHeight/9)));
            Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(26,98,161));
            Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
            Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2,28,OPT_CENTERX|OPT_CENTERY,"Storage Device not Found");
            Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
            Ft_Gpu_CoCmd_Swap(phost);
            Ft_App_Flush_Co_Buffer(phost);
            Ft_Gpu_Hal_WaitCmdfifo_empty(phost);	
            delay(5000);       
        }    
       else
      { 
           Serial.begin(9600);      
           Info();
           while(1)
           {
             Logo_Intial_setup(Main_Icons,10);
       	     Val =  MainWindow();
             if(Val == 'P')
      	     {
	        Play_Sound(0x56,SoundLev);
      	        WashWindow();
      	     }
           }
      } 
      Ft_Gpu_Hal_Close(phost);
      Ft_Gpu_Hal_DeInit();
#ifdef MSVC_PLATFORM
      return 0;
#endif
}

void loop()
{
}


/* Nothing beyond this */














