/* Sample application to demonstrat FT800 primitives, widgets and customized screen shots */

/* Copyright (c) Future Technology Devices International 2014

THIS SOFTWARE IS PROVIDED BY FUTURE TECHNOLOGY DEVICES INTERNATIONAL LIMITED "AS IS"

AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
FUTURE TECHNOLOGY DEVICES INTERNATIONAL LIMITED BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES LOSS OF USE, DATA, OR PROFITS OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

FTDI DRIVERS MAY BE USED ONLY IN CONJUNCTION WITH PRODUCTS BASED ON FTDI PARTS.

FTDI DRIVERS MAY BE DISTRIBUTED IN ANY FORM AS LONG AS LICENSE INFORMATION IS NOT MODIFIED.

IF A CUSTOM VENDOR ID AND/OR PRODUCT ID OR DESCRIPTION STRING ARE USED, IT IS THE
RESPONSIBILITY OF THE PRODUCT MANUFACTURER TO MAINTAIN ANY CHANGES AND SUBSEQUENT WHQL
RE-CERTIFICATION AS A RESULT OF MAKING THESE CHANGES.

Abstract:

This file contains is functions for all UI fields.

Author : FTDI */


#include "FT_Platform.h"
#ifdef FT900_PLATFORM
#include "ff.h"
#endif
#define SAMAPP_DELAY_BTW_APIS (1000)
#define SAMAPP_ENABLE_DELAY() Ft_Gpu_Hal_Sleep(SAMAPP_DELAY_BTW_APIS)
#define SAMAPP_ENABLE_DELAY_VALUE(x) Ft_Gpu_Hal_Sleep(x)
#define F16(s)        ((ft_int32_t)((s) * 65536))
#define SQ(v) (v*v)
#define MEMCMD(a) Ft_Gpu_Hal_WrCmdBuf(phost,a,sizeof(a))
#define NAMEARRAYSZ 14

/* Global variables for display resolution to support various display panels */
/* Default is WQVGA - 480x272 */
ft_int16_t FT_DispWidth = 480;
ft_int16_t FT_DispHeight = 272;
ft_int16_t FT_DispHCycle =  548;
ft_int16_t FT_DispHOffset = 43;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 41;
ft_int16_t FT_DispVCycle = 292;
ft_int16_t FT_DispVOffset = 12;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 10;
ft_uint8_t FT_DispPCLK = 5;
ft_char8_t FT_DispSwizzle = 0;
ft_char8_t FT_DispPCLKPol = 1;
ft_char8_t FT_DispCSpread = 1;
ft_char8_t FT_DispDither = 1;

/* Global used for buffer optimization */
Ft_Gpu_Hal_Context_t host,*phost;

#ifdef FT900_PLATFORM

FATFS FatFs;

FATFS            FatFs;				// FatFs work area needed for each volume
FIL 			 CurFile;
FRESULT          fResult;
SDHOST_STATUS    SDHostStatus;
#endif

ft_int16_t CartX = 480;

ft_uint32_t Ft_CmdBuffer_Index;
ft_uint32_t Ft_DlBuffer_Index;

#ifdef BUFFER_OPTIMIZATION
ft_uint8_t  Ft_DlBuffer[FT_DL_SIZE];
ft_uint8_t  Ft_CmdBuffer[FT_CMD_FIFO_SIZE];
#endif

/* Initial boot up DL - make the back ground green color */
const ft_uint8_t FT_DLCODE_BOOTUP[12] =
{
  255,255,255,2,//GPU instruction CLEAR_COLOR_RGB
  7,0,0,38, //GPU instruction CLEAR
  0,0,0,0,  //GPU instruction DISPLAY
};

#if defined(FT900_PLATFORM)
DWORD get_fattime(void){
	return 0;
}
#endif

ft_void_t Ft_App_WrCoCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef  BUFFER_OPTIMIZATION
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_CmdBuffer[Ft_CmdBuffer_Index];
   *pBuffcmd = cmd;
#endif
#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif
#ifdef FT900_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif

   /* Increment the command index */
   Ft_CmdBuffer_Index += FT_CMD_SIZE;
}

ft_void_t Ft_App_WrDlCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef BUFFER_OPTIMIZATION  
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_DlBuffer[Ft_DlBuffer_Index];
   *pBuffcmd = cmd;
#endif

#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
#ifdef FT900_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
   /* Increment the command index */
   Ft_DlBuffer_Index += FT_CMD_SIZE;
}

ft_void_t Ft_App_WrCoStr_Buffer(Ft_Gpu_Hal_Context_t *phost,const ft_char8_t *s)
{
#ifdef  BUFFER_OPTIMIZATION  
  ft_uint16_t length = 0;
  length = strlen(s) + 1;//last for the null termination

  strcpy(&Ft_CmdBuffer[Ft_CmdBuffer_Index],s);

  /* increment the length and align it by 4 bytes */
  Ft_CmdBuffer_Index += ((length + 3) & ~3);
#endif  
}

ft_void_t Ft_App_Flush_DL_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_DlBuffer_Index> 0)
     Ft_Gpu_Hal_WrMem(phost,RAM_DL,Ft_DlBuffer,Ft_DlBuffer_Index);
#endif     
   Ft_DlBuffer_Index = 0;

}

ft_void_t Ft_App_Flush_Co_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_CmdBuffer_Index > 0)
     Ft_Gpu_Hal_WrCmdBuf(phost,Ft_CmdBuffer,Ft_CmdBuffer_Index);
#endif     
   Ft_CmdBuffer_Index = 0;
}


/* API to give fadeout effect by changing the display PWM from 100 till 0 */
ft_void_t SAMAPP_fadeout()
{
   ft_int32_t i;

	for (i = 100; i >= 0; i -= 3)
	{
		Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);

		Ft_Gpu_Hal_Sleep(2);//sleep for 2 ms
	}
}

/* API to perform display fadein effect by changing the display PWM from 0 till 100 and finally 128 */
ft_void_t SAMAPP_fadein()
{
	ft_int32_t i;

	for (i = 0; i <=100 ; i += 3)
	{
		Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);
		Ft_Gpu_Hal_Sleep(2);//sleep for 2 ms
	}
	/* Finally make the PWM 100% */
	i = 128;
	Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);
}


/* API to check the status of previous DLSWAP and perform DLSWAP of new DL */
/* Check for the status of previous DLSWAP and if still not done wait for few ms and check again */
ft_void_t SAMAPP_GPU_DLSwap(ft_uint8_t DL_Swap_Type)
{
	ft_uint8_t Swap_Type = DLSWAP_FRAME,Swap_Done = DLSWAP_FRAME;

	if(DL_Swap_Type == DLSWAP_LINE)
	{
		Swap_Type = DLSWAP_LINE;
	}

	/* Perform a new DL swap */
	Ft_Gpu_Hal_Wr8(phost,REG_DLSWAP,Swap_Type);

	/* Wait till the swap is done */
	while(Swap_Done)
	{
		Swap_Done = Ft_Gpu_Hal_Rd8(phost,REG_DLSWAP);

		if(DLSWAP_DONE != Swap_Done)
		{
			Ft_Gpu_Hal_Sleep(10);//wait for 10ms
		}
	}
}

#ifdef FT900_PLATFORM
ft_void_t FT900_Config()
{
	sys_enable(sys_device_uart0);
    gpio_function(48, pad_uart0_txd); /* UART0 TXD */
    gpio_function(49, pad_uart0_rxd); /* UART0 RXD */
    uart_open(UART0,                    /* Device */
              1,                        /* Prescaler = 1 */
              UART_DIVIDER_115200_BAUD,  /* Divider = 1302 */
              uart_data_bits_8,         /* No. Data Bits */
              uart_parity_none,         /* Parity */
              uart_stop_bits_1);        /* No. Stop Bits */

    /* Print out a welcome message... */
    uart_puts(UART0,

        "(C) Copyright 2014-2015, Future Technology Devices International Ltd. \r\n"
        "--------------------------------------------------------------------- \r\n"
        "Welcome to WashingMach Example ... \r\n"
        "\r\n"
        "--------------------------------------------------------------------- \r\n"
        );

#ifdef ENABLE_ILI9488_HVGA_PORTRAIT
/* asign all the respective pins to gpio and set them to default values */
gpio_function(34, pad_gpio34);
gpio_dir(34, pad_dir_output);
gpio_write(34,1);

gpio_function(27, pad_gpio27);
gpio_dir(27, pad_dir_output);
gpio_write(27,1);

gpio_function(29, pad_gpio29);
gpio_dir(29, pad_dir_output);
gpio_write(29,1);

gpio_function(33, pad_gpio33);
gpio_dir(33, pad_dir_output);
gpio_write(33,1);


gpio_function(30, pad_gpio30);
gpio_dir(30, pad_dir_output);
gpio_write(30,1);

gpio_function(28, pad_gpio28);
gpio_dir(28, pad_dir_output);
gpio_write(28,1);


	gpio_function(43, pad_gpio43);
	gpio_dir(43, pad_dir_output);
gpio_write(43,1);
gpio_write(34,1);
gpio_write(28,1);
gpio_write(43,1);
gpio_write(33,1);
gpio_write(33,1);

#endif
/*use for sd card*/
sys_enable(sys_device_sd_card);
	//	sdhost_sys_init();
		sdhost_init();

			#define GPIO_SD_CLK  (19)
			#define GPIO_SD_CMD  (20)
			#define GPIO_SD_DAT3 (21)
			#define GPIO_SD_DAT2 (22)
			#define GPIO_SD_DAT1 (23)
			#define GPIO_SD_DAT0 (24)
			#define GPIO_SD_CD   (25)
			#define GPIO_SD_WP   (26)
		    gpio_function(GPIO_SD_CLK, pad_sd_clk); gpio_pull(GPIO_SD_CLK, pad_pull_none);//pad_pull_none
		    gpio_function(GPIO_SD_CMD, pad_sd_cmd); gpio_pull(GPIO_SD_CMD, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT3, pad_sd_data3); gpio_pull(GPIO_SD_DAT3, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT2, pad_sd_data2); gpio_pull(GPIO_SD_DAT2, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT1, pad_sd_data1); gpio_pull(GPIO_SD_DAT1, pad_pull_pullup);
		    gpio_function(GPIO_SD_DAT0, pad_sd_data0); gpio_pull(GPIO_SD_DAT0, pad_pull_pullup);
		    gpio_function(GPIO_SD_CD, pad_sd_cd); gpio_pull(GPIO_SD_CD, pad_pull_pullup);
		    gpio_function(GPIO_SD_WP, pad_sd_wp); gpio_pull(GPIO_SD_WP, pad_pull_pullup);
		    SDHOST_STATUS sd_status;
		  	if(sdhost_card_detect() == SDHOST_CARD_INSERTED)
		  		{
		  		printf("\n sd_status=%s",sdhost_card_detect());
		  		}

		  	if (sdhost_card_detect() != SDHOST_CARD_INSERTED)
		  		{
		  		printf("Please Insert SD Card\r\n");
		  		}
		  	else
		  		{printf("\n SD Card inserted!");}


		  	if (f_mount(&FatFs, "", 0) != FR_OK)
		  		{
		  		printf("\n Mounted failed.");;
		  		}

		  		printf("\n Mounted succesfully.");
/* useful for timer */
ft_millis_init();
interrupt_enable_globally();
//printf("ft900 config done \n");
}
#endif

ft_void_t Ft_BootupConfig()
{
	Ft_Gpu_Hal_Powercycle(phost,FT_TRUE);

		/* Access address 0 to wake up the FT800 */
		Ft_Gpu_HostCommand(phost,FT_GPU_ACTIVE_M);
		Ft_Gpu_Hal_Sleep(20);

		/* Set the clk to external clock */
#if (!defined(ME800A_HV35R) && !defined(ME810A_HV35R) && !defined(ME812AU_WH50R) && !defined(ME813AU_WH50C) && !defined(ME810AU_WH70R) && !defined(ME811AU_WH70C))
		Ft_Gpu_HostCommand(phost,FT_GPU_EXTERNAL_OSC);
		Ft_Gpu_Hal_Sleep(10);
#endif

		{
			ft_uint8_t chipid;
			//Read Register ID to check if FT800 is ready.
			chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
			while(chipid != 0x7C)
			{
				chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
				ft_delay(100);
			}
	#if defined(MSVC_PLATFORM) || defined (FT900_PLATFORM)
			printf("VC1 register ID after wake up %x\n",chipid);
	#endif
	}
	/* Configuration of LCD display */
#ifdef DISPLAY_RESOLUTION_QVGA
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 320;
	FT_DispHeight = 240;
	FT_DispHCycle =  408;
	FT_DispHOffset = 70;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 263;
	FT_DispVOffset = 13;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 2;
	FT_DispPCLK = 8;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 0;
	FT_DispCSpread = 1;
	FT_DispDither = 1;

#endif
#ifdef DISPLAY_RESOLUTION_WVGA
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 800;
	FT_DispHeight = 480;
	FT_DispHCycle =  928;
	FT_DispHOffset = 88;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 48;
	FT_DispVCycle = 525;
	FT_DispVOffset = 32;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 3;
	FT_DispPCLK = 2;
	FT_DispSwizzle = 0;
	FT_DispPCLKPol = 1;
	FT_DispCSpread = 0;
	FT_DispDither = 1;
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
	/* Values specific to HVGA LCD display */

	FT_DispWidth = 320;
	FT_DispHeight = 480;
	FT_DispHCycle =  400;
	FT_DispHOffset = 40;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 500;
	FT_DispVOffset = 10;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 5;
	FT_DispPCLK = 4;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 1;
	FT_DispCSpread = 1;
	FT_DispDither = 1;

#ifdef ME810A_HV35R
	FT_DispPCLK = 5;
#endif

#endif

#if defined(ME800A_HV35R)
	/* After recognizing the type of chip, perform the trimming if necessary */
    Ft_Gpu_ClockTrimming(phost,LOW_FREQ_BOUND);
#endif

	Ft_Gpu_Hal_Wr16(phost, REG_HCYCLE, FT_DispHCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_HOFFSET, FT_DispHOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC0, FT_DispHSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_HSYNC1, FT_DispHSync1);
	Ft_Gpu_Hal_Wr16(phost, REG_VCYCLE, FT_DispVCycle);
	Ft_Gpu_Hal_Wr16(phost, REG_VOFFSET, FT_DispVOffset);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC0, FT_DispVSync0);
	Ft_Gpu_Hal_Wr16(phost, REG_VSYNC1, FT_DispVSync1);
	Ft_Gpu_Hal_Wr8(phost, REG_SWIZZLE, FT_DispSwizzle);
	Ft_Gpu_Hal_Wr8(phost, REG_PCLK_POL, FT_DispPCLKPol);
	Ft_Gpu_Hal_Wr16(phost, REG_HSIZE, FT_DispWidth);
	Ft_Gpu_Hal_Wr16(phost, REG_VSIZE, FT_DispHeight);
	Ft_Gpu_Hal_Wr16(phost, REG_CSPREAD, FT_DispCSpread);
	Ft_Gpu_Hal_Wr16(phost, REG_DITHER, FT_DispDither);

#if (defined(FT_800_ENABLE) || defined(FT_810_ENABLE) ||defined(FT_812_ENABLE))
    /* Touch configuration - configure the resistance value to 1200 - this value is specific to customer requirement and derived by experiment */
    Ft_Gpu_Hal_Wr16(phost, REG_TOUCH_RZTHRESH,RESISTANCE_THRESHOLD);
#endif
    #if defined(FT_81X_ENABLE)
        Ft_Gpu_Hal_Wr16(phost, REG_GPIOX_DIR, 0xffff);
        Ft_Gpu_Hal_Wr16(phost, REG_GPIOX, 0xffff); 
    #else
        Ft_Gpu_Hal_Wr8(phost, REG_GPIO_DIR, 0xff);
        Ft_Gpu_Hal_Wr8(phost, REG_GPIO, 0xff); 
    #endif

    /*It is optional to clear the screen here*/
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
    Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);


    Ft_Gpu_Hal_Wr8(phost, REG_PCLK,FT_DispPCLK);//after this display is visible on the LCD


#ifdef ENABLE_ILI9488_HVGA_PORTRAIT
	/* to cross check reset pin */
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);
	ft_delay(120);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0x7f);
	ft_delay(120);
	Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0xff);

	ILI9488_Bootup();

	/* Reconfigure the SPI */
#ifdef FT900_PLATFORM
	printf("after ILI9488 bootup \n");
	//spi
	// Initialize SPIM HW
	sys_enable(sys_device_spi_master);
	gpio_function(27, pad_spim_sck); /* GPIO27 to SPIM_CLK */
	gpio_function(28, pad_spim_ss0); /* GPIO28 as CS */
	gpio_function(29, pad_spim_mosi); /* GPIO29 to SPIM_MOSI */
	gpio_function(30, pad_spim_miso); /* GPIO30 to SPIM_MISO */

	gpio_write(28, 1);
	spi_init(SPIM, spi_dir_master, spi_mode_0, 4);
#endif

#endif



	/* make the spi to quad mode - addition 2 bytes for silicon */
#ifdef FT_81X_ENABLE
	/* api to set quad and numbe of dummy bytes */
#ifdef ENABLE_SPI_QUAD
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_QUAD_CHANNEL,FT_GPU_SPI_TWODUMMY);
#elif ENABLE_SPI_DUAL
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_DUAL_CHANNEL,FT_GPU_SPI_TWODUMMY);
#else
	Ft_Gpu_Hal_SetSPI(phost,FT_GPU_SPI_SINGLE_CHANNEL,FT_GPU_SPI_ONEDUMMY);
#endif

#endif

#ifdef FT900_PLATFORM
    /* Change clock frequency to 25mhz */
	spi_init(SPIM, spi_dir_master, spi_mode_0, 4);

#if (defined(ENABLE_SPI_QUAD))
    /* Initialize IO2 and IO3 pad/pin for dual and quad settings */
    gpio_function(31, pad_spim_io2);
    gpio_function(32, pad_spim_io3);
    gpio_write(31, 1);
    gpio_write(32, 1);
#endif
	/* Enable FIFO of QSPI */
	spi_option(SPIM,spi_option_fifo_size,64);
	spi_option(SPIM,spi_option_fifo,1);
	spi_option(SPIM,spi_option_fifo_receive_trigger,1);
#endif

#ifdef ENABLE_SPI_QUAD
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,4);
#endif
#elif ENABLE_SPI_DUAL	
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,2);
#endif
#else
#ifdef FT900_PLATFORM
	spi_option(SPIM,spi_option_bus_width,1);
#endif

	
#endif


phost->ft_cmd_fifo_wp = Ft_Gpu_Hal_Rd16(phost,REG_CMD_WRITE);

}





/*****************************************************************************/
/* Example code to display few points at various offsets with various colors */




char *info[] = {
					  "FT800 Washing Machine Demo Application",
          			  "APP to demonstrate interactive Menu,",
					  "and animation,",
					  "using Bitmaps",
					 };



float linear(float p1,float p2,ft_uint16_t t,ft_uint16_t rate)
{
	float st  = (float)t/rate;
	return p1+(st*(p2-p1));
}

ft_uint16_t smoothstep(ft_int16_t p1,ft_int16_t p2,ft_uint16_t t,ft_uint16_t rate)
{
	float dst  = (float)t/rate;
	float st = SQ(dst)*(3-2*dst);
	return p1+(st*(p2-p1));
}

ft_uint16_t acceleration(ft_uint16_t p1,ft_uint16_t p2,ft_uint16_t t,ft_uint16_t rate)
{
	float dst  = (float)t/rate;
	float st = SQ(dst);
	return p1+(st*(p2-p1));
}

float deceleration(ft_uint16_t p1,ft_uint16_t p2,ft_uint16_t t,ft_uint16_t rate)
{
	float st,dst  = (float)t/rate;
	dst = 1-dst;
	st = 1-SQ(dst);
	return p1+(st*(p2-p1));
}



static ft_uint8_t key_pressed ;

ft_uint8_t Read_Keys()
{
	static ft_uint8_t Read_tag=0,temp_tag=0,ret_tag=0;
	ret_tag = NULL;
	Read_tag = Ft_Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);
	if(Read_tag!=NULL)											// Allow if the Key is released
	{
		if(temp_tag!=Read_tag)
		{
			temp_tag = Read_tag;
			key_pressed = Read_tag;										// Load the Read tag to temp variable
		}
	}
	else
	{
		key_pressed = 0;
		if(temp_tag!=0)
		{
			ret_tag = temp_tag;
			temp_tag = 0;
		}
	}
	return ret_tag;
}

static ft_uint8_t istouch()
{
  return !(Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_RAW_XY) & 0x8000);
}
void Use_Rgb()
{
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));
}
void Use_Alpha()
{
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
}

void Clear_Alpha()
{
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(0));
}
void end_frame()
{
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
}


#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
void Load_file2gram(ft_uint32_t add,ft_uint8_t sectors, FILE *afile)
{
  ft_uint8_t pbuff[512],temp[512],tval;
  ft_uint16_t z = 0;
  for(z=0;z<sectors;z++)
  {
	fread(pbuff,1,512,afile);
	Ft_Gpu_Hal_WrMem(phost,add,pbuff,512L);
	add+=512;
  }
}
#endif

ft_void_t Play_Sound(ft_uint8_t sound,ft_uint8_t volume)
{
		  Ft_Gpu_Hal_Wr8(phost,REG_VOL_SOUND,volume);
		  Ft_Gpu_Hal_Wr8(phost,REG_SOUND,sound);
		  Ft_Gpu_Hal_Wr8(phost,REG_PLAY,1);
}



ft_void_t Info()
{
		  ft_uint16_t dloffset = 0,z=0;
		  Ft_CmdBuffer_Index = 0;
		  Ft_Gpu_CoCmd_Dlstart(phost);
		  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
		  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2,29,OPT_CENTERX|OPT_CENTERY,"Please tap on a dot");
		  Ft_Gpu_CoCmd_Calibrate(phost,0);
		  //Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
		  //Ft_Gpu_CoCmd_Swap(phost);
		  Ft_App_Flush_Co_Buffer(phost);
		  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

	  Ft_Gpu_CoCmd_Logo(phost);
	  Ft_App_Flush_Co_Buffer(phost);
	  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

	  while(0!=Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ));
	  dloffset = Ft_Gpu_Hal_Rd16(phost,REG_CMD_DL);
	  dloffset -=4;

#ifdef FT_81X_ENABLE
  dloffset -= 2*4;//remove two more instructions in case of 81x
#endif

	  Ft_Gpu_Hal_WrCmd32(phost,CMD_MEMCPY);
	  Ft_Gpu_Hal_WrCmd32(phost,100000L);
	  Ft_Gpu_Hal_WrCmd32(phost,RAM_DL);
	  Ft_Gpu_Hal_WrCmd32(phost,dloffset);

	do
	{
			Ft_Gpu_CoCmd_Dlstart(phost);
			Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
			Ft_Gpu_CoCmd_Append(phost,100000L,dloffset);
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_B(0));
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_C(0));
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_D(0));
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_F(0));
			Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(220));
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,FT_DispHeight*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
			Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
		   // INFORMATION
#ifdef DISPLAY_RESOLUTION_WVGA
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,40,30,OPT_CENTERX|OPT_CENTERY,info[0]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,80,29,OPT_CENTERX|OPT_CENTERY,info[1]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,120,29,OPT_CENTERX|OPT_CENTERY,info[2]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,160,29,OPT_CENTERX|OPT_CENTERY,info[3]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight-30,29,OPT_CENTERX|OPT_CENTERY,"Click to play");
#else
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,20,28,OPT_CENTERX|OPT_CENTERY,info[0]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,60,26,OPT_CENTERX|OPT_CENTERY,info[1]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,90,26,OPT_CENTERX|OPT_CENTERY,info[2]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,120,26,OPT_CENTERX|OPT_CENTERY,info[3]);
			Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight-30,26,OPT_CENTERX|OPT_CENTERY,"Click to play");

#endif
			if(key_pressed!='P')
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
			else
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(100,100,100));

			Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
			Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16));
			Ft_App_WrCoCmd_Buffer(phost,TAG('P'));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth/2)*16,(FT_DispHeight-60)*16));

			//create a triangle
			Ft_App_WrCoCmd_Buffer(phost,CLEAR(0,1,0));
			Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 0, 0) );
			Ft_App_WrCoCmd_Buffer(phost, STENCIL_OP(INCR,INCR) );
			Ft_App_WrCoCmd_Buffer(phost, COLOR_MASK(0,0,0,0) );//mask all the colors
			Ft_App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_L));

			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-77+5)*16));
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-2+15)*16,(FT_DispHeight-67+8)*16));
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-57+11)*16));
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-77+5)*16));

			Ft_App_WrCoCmd_Buffer(phost, COLOR_MASK(1,1,1,1) );//enable all the colors
			Ft_App_WrCoCmd_Buffer(phost, STENCIL_FUNC(EQUAL,1,255));
			Ft_App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_L));
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(FT_DispWidth*16,0));
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));

			Ft_App_WrCoCmd_Buffer(phost, STENCIL_OP(KEEP,KEEP));
			Ft_App_WrCoCmd_Buffer(phost, STENCIL_FUNC(ALWAYS,0,255));
			Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));

			Ft_App_WrCoCmd_Buffer(phost, BEGIN(LINE_STRIP));
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-77+5)*16));
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-2+15)*16,(FT_DispHeight-67+8)*16));
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-57+11)*16));
			Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(((FT_DispWidth/2)-14+10)*16,(FT_DispHeight-77+5)*16));

			Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
			Ft_Gpu_CoCmd_Swap(phost);
			Ft_App_Flush_Co_Buffer(phost);
			Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

		}while(Read_Keys()!='P');
#if 0

	  ft_uint16_t dloffset = 0,z;
	  Ft_CmdBuffer_Index = 0;

	// Touch Screen Calibration

	  Ft_Gpu_CoCmd_Dlstart(phost);
	  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
	  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
	  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2,26,OPT_CENTERX|OPT_CENTERY,"Please tap on a dot");
	  Ft_Gpu_CoCmd_Calibrate(phost,0);
	  Ft_App_Flush_Co_Buffer(phost);
	  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

	  Ft_Gpu_CoCmd_Logo(phost);
	  Ft_App_Flush_Co_Buffer(phost);
	  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
	  while(0!=Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ));
	  dloffset = Ft_Gpu_Hal_Rd16(phost,REG_CMD_DL);
	  dloffset -=4;
	  Ft_Gpu_Hal_WrCmd32(phost,CMD_MEMCPY);
	  Ft_Gpu_Hal_WrCmd32(phost,100000L);
	  Ft_Gpu_Hal_WrCmd32(phost,RAM_DL);
	  Ft_Gpu_Hal_WrCmd32(phost,dloffset);
	  do
	  {
	    Ft_Gpu_CoCmd_Dlstart(phost);
	    //Ft_Gpu_CoCmd_Append(phost,100000L,dloffset);
	    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
	    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_B(0));
	    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_C(0));
	    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_D(0));
	    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
	    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_F(0));
	    //Display the information with transparent Logo using Edge Strip
	    Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
	    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
	    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(220));
	    Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
	    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,FT_DispHeight*16));
	    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));
	    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
	    Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
	    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
	   // INFORMATION
	    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,20,28,OPT_CENTERX|OPT_CENTERY,(char*)(info[0]));
	    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,60,26,OPT_CENTERX|OPT_CENTERY,(char*)(info[1]));
	    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,90,26,OPT_CENTERX|OPT_CENTERY,(char*)(info[2]));
	    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,120,26,OPT_CENTERX|OPT_CENTERY,(char*)(info[3]));
	    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight-30,26,OPT_CENTERX|OPT_CENTERY,"Click to play");
	    //Check if the Play key and change the color
	    if(key_pressed!='P')
	    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
	    else
	    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(100,100,100));
	    Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
	    Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16));
	    Ft_App_WrCoCmd_Buffer(phost,TAG('P'));
	    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth/2)*16,(FT_DispHeight-60)*16));
	    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(180,35,35));
	    //Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
	    //Ft_App_WrCoCmd_Buffer(phost,VERTEX2II((FT_DispWidth/2)-14,(FT_DispHeight-75),14,4));
	    Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
	    Ft_Gpu_CoCmd_Swap(phost);
	    Ft_App_Flush_Co_Buffer(phost);
	    Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
	  }while(Read_Keys()!='P');
#endif

		  Ft_Gpu_CoCmd_Dlstart(phost);
		  Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(255,255,255));
		  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
		  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
		  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2+40,29,OPT_CENTERX|OPT_CENTERY,"Data Loading");
#endif
#if defined(DISPLAY_RESOLUTION_WQVGA)
		  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2-80,FT_DispHeight/2,29,OPT_CENTERX|OPT_CENTERY,"Data Loading");
#endif
#if defined(DISPLAY_RESOLUTION_WVGA)
		  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2-120,FT_DispHeight/2,29,OPT_CENTERX|OPT_CENTERY,"Data Loading");
#endif
		  Ft_Gpu_CoCmd_Spinner(phost,FT_DispWidth/2+40,FT_DispHeight/2,1,0);
		  //Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
		  //Ft_Gpu_CoCmd_Swap(phost);
		  Ft_App_Flush_Co_Buffer(phost);
		  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
		  Play_Sound(0x50,255);

#if defined(FT_801_ENABLE) || defined(FT_811_ENABLE) || defined(FT_813_ENABLE)
		  Ft_Gpu_Hal_Wr8(phost,REG_CTOUCH_EXTENDED, CTOUCH_MODE_EXTENDED);
#endif
}

static struct {
  signed short dragprev;
  int vel;      // velocity
  long base;    // screen x coordinate, in 1/16ths pixel
  long limit;
} scroller;

static void scroller_init(ft_uint32_t limit)
{
  scroller.dragprev = -32768;
  scroller.vel = 0;      // velocity
  scroller.base = 0;     // screen x coordinate, in 1/16ths pixel
  scroller.limit = limit;
}

/*
static void scroller_run()
{
  signed short sy = Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);

  if ((sy != -32768) & (scroller.dragprev != -32768)) {
    scroller.vel = (scroller.dragprev - sy) << 4;
  } else {
    int change = max(1, abs(scroller.vel) >> 5);
    if (scroller.vel < 0)
      scroller.vel += change;
    if (scroller.vel > 0)
      scroller.vel -= change;
  }
  scroller.dragprev = sy;

  scroller.base += scroller.vel;

  scroller.base = max(0, min(scroller.base, scroller.limit));
}
*/

static ft_uint16_t scroller_run(ft_uint8_t touchTag)
{
  signed short sy=-32768;

if(touchTag==101){
  sy = Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);
  if((sy != -32768) && (scroller.dragprev == -32768)){
	  scroller.dragprev = sy;
  }
  else if ((sy != -32768) & (scroller.dragprev != -32768)) {
    if(scroller.dragprev > sy)
    	scroller.vel = scroller.dragprev - sy;
    else if(scroller.dragprev < sy)
    	scroller.vel = scroller.dragprev - sy;

    scroller.dragprev = sy;

    scroller.base += (scroller.vel/2);

    scroller.base = max(0, min(scroller.base, scroller.limit));


  }
}
else{
	scroller.dragprev = -32768;
}

return scroller.base;
}

 typedef struct logoim
{
  //ft_prog_uchar8_t name[14];
  ft_char8_t name[NAMEARRAYSZ];
  ft_uint16_t image_height;
  ft_uint8_t image_format;
  ft_uint8_t filter;
  ft_uint16_t sizex;
  ft_uint16_t sizey;
  ft_uint16_t linestride;
  ft_uint32_t gram_address;
}t_imageprp;



#ifdef DISPLAY_RESOLUTION_WQVGA
FT_PROGMEM t_imageprp Main_Icons[10] =
{
	{"shirt.bin",		 40,    ARGB4,  NEAREST,    40,     40 ,    80,   0L      },
	{"Temp.bin",		 40,    ARGB4,  NEAREST,    40,     40 ,    80,   3L      },
	{"SpinW.bin",        40,    ARGB4,  NEAREST,    40,     40 ,    80,   7L      },
	{"Butfly.bin",	     20,    ARGB4,  NEAREST,    20,     20 ,    40,   11L     },
	{"Lock.bin",		 30,    ARGB4,  NEAREST,    30,     30 ,    60,   12L     },
	{"UnLock.bin",		 30,    ARGB4,  NEAREST,    30,     60 ,    60,   14L     },
	{"Set.bin",		     30,    ARGB4,  NEAREST,    30,     30 ,    60,   16L     },
	{"Start.bin",		 30,    ARGB4,  NEAREST,    30,     30 ,    60,   18L     },
	{"home.bin",		 40,    ARGB4,  NEAREST,    40,     40 ,    80,   20L     },//4kb
	{"Bkgd.bin",        136,    L8,     NEAREST,    480,   272 ,   240,   24L     },
},

Wash_Window[7] =
{
	{"Wash.bin",		55,	 ARGB4,	NEAREST,    50,	   50 ,	  100,	 57L	}, // size 22KB
	{"Rinse.bin",		54,	 ARGB4,	NEAREST,    54,	   54 ,	  108,	 79L	}, // size 24KB
	{"Spin.bin",		54,	 ARGB4,	NEAREST,    45,	   45 ,	   90,	103L	}, // size 30KB
	{"WBar.bin",		52,	RGB565,	NEAREST,   400,	  400 ,	  800,	133L	}, //size 42KB
	{"bub.bin",		    30,	 ARGB4,	NEAREST,    30,	   30 ,	   60,	175L	},//2kb
	{"home.bin",		40,	 ARGB4,	NEAREST,    40,	   40 ,	   80,	177L	},//4kb
	{"Bkgd.bin",		136,    L8,	NEAREST,   480,	  272 ,	  240,	181L	},   //33kb
};
#elif DISPLAY_RESOLUTION_WVGA

#ifdef FT900_PLATFORM
	t_imageprp Main_Icons[10] =
#else
	FT_PROGMEM t_imageprp Main_Icons[10] =
#endif
{
	{"shirtH.bin",		 70,    ARGB4,  NEAREST,    70,    70 ,    140,   0},   //10
	{"TempH.bin",		 70,    ARGB4,  NEAREST,    70,     70 ,    140,   10},  //10
	{"SpinWH.bin",       70,    ARGB4,  NEAREST,    70,     70 ,    140,   20},  //10
	{"ButflyH.bin",	     26,    ARGB4,  NEAREST,    26,     26 ,    52,   30},     //2
	{"LockH.bin",		 54,    ARGB4,  NEAREST,    54,     54 ,    108,   32},  //6
	{"UnLockH.bin",		 53,    ARGB4,  NEAREST,    66,     53 ,    132,   38},  //7
	{"SetH.bin",		 52,    ARGB4,  NEAREST,    52,     52 ,    104,   45     },  //6
	{"StartH.bin",		 52,    ARGB4,  NEAREST,    52,     52 ,    104,   51     },  //6
	{"homeH.bin",		 70,    ARGB4,  NEAREST,    70,     70 ,    140,   57     }, //10
	{"BkgdH.bin",        240,      L8,  NEAREST,   800,    480 ,    400,   67     },         //94
};

#ifdef FT900_PLATFORM
	t_imageprp Wash_Window[7] =
#else
		FT_PROGMEM	t_imageprp  Wash_Window[7] =
#endif
{
	{"WashH.bin",		88,	 ARGB4,	NEAREST,    88,	   352,	  176,	161	}, // 61
	{"RinseH.bin",		96,	 ARGB4,	NEAREST,    96,	   384,	  192,	222	}, // 72
	{"SpinH.bin",		80,	 ARGB4,	NEAREST,    80,	   480,	  160,	294	}, // 75
	{"WBarH.bin",		94,	RGB565,	NEAREST,   664,	   94 ,	  1328,	369	}, //122
	{"bubH.bin",		54,	 ARGB4,	NEAREST,    54,	   54 ,	  108,	491	}, //6
	{"homeH.bin",		70,	 ARGB4,	NEAREST,    70,	   70 ,	  140,	497	}, //10
	{"BkgdH.bin",	   240,    L8,	NEAREST,   800,	  480 ,	  400,	591	}, //94
};

#endif

	ft_uint8_t Stflag,OptionsSet,MainFlag,Tagval,Pf=0;
	ft_uint16_t BrightNess,SoundLev=255,TimeSet,IterCount,Px;
#ifdef DISPLAY_RESOLUTION_WQVGA
	static ft_uint16_t ButX=330;
#elif DISPLAY_RESOLUTION_WVGA
	static ft_uint16_t ButX=592;
#endif



#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
 t_imageprp Main_Icons[10] =
{
	{"shirt.bin",		 40,    ARGB4,  NEAREST,    40,     40 ,    80,   0L      },
	{"Temp.bin",		 40,    ARGB4,  NEAREST,    40,     40 ,    80,   3L      },
	{"SpinW.bin",        40,    ARGB4,  NEAREST,    40,     40 ,    80,   7L      },
	{"Butfly.bin",	     20,    ARGB4,  NEAREST,    20,     20 ,    40,   11L     },
	{"Lock.bin",		 30,    ARGB4,  NEAREST,    30,     30 ,    60,   12L     },
	{"UnLock.bin",		 30,    ARGB4,  NEAREST,    30,     60 ,    60,   14L     },
	{"Set.bin",		     30,    ARGB4,  NEAREST,    30,     30 ,    60,   16L     },
	{"Start.bin",		 30,    ARGB4,  NEAREST,    30,     30 ,    60,   18L     },
	{"home.bin",		 40,    ARGB4,  NEAREST,    40,     40 ,    80,   20L     },//4kb
	{"Bkgd.bin",        136,    L8,     NEAREST,    320,   480 ,   240,   24L     },
},

Wash_Window[7] =
{
	{"Wash.bin",		55,	 ARGB4,	NEAREST,    50,	   50 ,	  100,	 57L	}, // size 22KB
	{"Rinse.bin",		54,	 ARGB4,	NEAREST,    54,	   54 ,	  108,	 79L	}, // size 24KB
	{"Spin.bin",		54,	 ARGB4,	NEAREST,    45,	   45 ,	   90,	103L	}, // size 30KB
	{"WBar.bin",		52,	RGB565,	NEAREST,   320,	  400 ,	  800,	133L	}, //size 42KB
	{"bub.bin",		    30,	 ARGB4,	NEAREST,    30,	   30 ,	   60,	175L	},//2kb
	{"home.bin",		40,	 ARGB4,	NEAREST,    40,	   40 ,	   80,	177L	},//4kb
	{"Bkgd.bin",		136,    L8,	NEAREST,   320,	  480 ,	  240,	181L	},   //33kb
};


#ifdef FT900_PLATFORM
	t_imageprp Wash_Window[7] =
#else
		FT_PROGMEM	t_imageprp  Wash_Window[7] =
#endif
{
	{"WashH.bin",		88,	 ARGB4,	NEAREST,    88,	   352,	  176,	161	}, // 61
	{"RinseH.bin",		96,	 ARGB4,	NEAREST,    96,	   384,	  192,	222	}, // 72
	{"SpinH.bin",		80,	 ARGB4,	NEAREST,    80,	   480,	  160,	294	}, // 75
	{"WBarH.bin",		94,	RGB565,	NEAREST,   664,	   94 ,	  1328,	369	}, //122
	{"bubH.bin",		54,	 ARGB4,	NEAREST,    54,	   54 ,	  108,	491	}, //6
	{"homeH.bin",		70,	 ARGB4,	NEAREST,    70,	   70 ,	  140,	497	}, //10
	{"BkgdH.bin",	   240,    L8,	NEAREST,   800,	  480 ,	  400,	591	}, //94
};

#endif

//	ft_uint8_t Stflag,OptionsSet,MainFlag,Tagval;
	//ft_uint16_t BrightNess,SoundLev=255,TimeSet,IterCount,Px;


#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
	static ft_uint16_t ButX=30;
#endif
	void load_inflate_image(ft_uint32_t address, const char *filename)
{
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	FILE *afile;
	ft_uint32_t ftsize=0;
	ft_uint8_t pbuff[8192];
	ft_uint16_t blocklen;

	Ft_Gpu_Hal_WrCmd32(phost,CMD_INFLATE);
	Ft_Gpu_Hal_WrCmd32(phost,address*1024L);
	afile = fopen(filename,"rb");		// read Binary (rb)
	if(afile==NULL)
		return;
	fseek(afile,0,SEEK_END);
	ftsize = ftell(afile);
	fseek(afile,0,SEEK_SET);
	while(ftsize>0)
	{
		blocklen = ftsize>8192?8192:ftsize;
		fread(pbuff,1,blocklen,afile);							/* copy the data into pbuff and then transfter it to command buffer */
		ftsize -= blocklen;
		Ft_Gpu_Hal_WrCmdBuf(phost,pbuff,blocklen);				/* copy data continuously into command memory */
	}
	fclose(afile);												/* close the opened jpg file */
#elif defined (FT900_PLATFORM)
	FIL InfSrc;

	ft_int32_t blocklen,filesize;
	ft_uint32_t g_random_buffer[512L];
	char tempName[14]="0:\\";



	strcat(tempName,filename);

	fResult = f_open(&InfSrc, filename, FA_READ | FA_OPEN_EXISTING);
	if(fResult==FR_OK)
	{
		Ft_Gpu_Hal_WrCmd32(phost,CMD_INFLATE);
		Ft_Gpu_Hal_WrCmd32(phost,address*1024L);
		filesize = f_size(&InfSrc);
		while(filesize>0)
		{
			fResult = f_read(&InfSrc, g_random_buffer, 512, &blocklen);	// read a chunk of src file
			filesize -= blocklen;
			Ft_Gpu_Hal_WrCmdBuf(phost,g_random_buffer,blocklen);

		}
		f_close(&InfSrc);

	}
	else{
		printf("Unable to open: %s\n",tempName);
	}

#elif defined (ARDUINO_PLATFORM)

#endif
}



void Logo_Intial_setup(struct logoim sptr[],ft_uint8_t num)
{
	ft_uint8_t z;

   for(z=0;z<num;z++)
   {
     load_inflate_image(sptr[z].gram_address,sptr[z].name);
   }

   Ft_Gpu_CoCmd_Dlstart(phost);        // start
   Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
   for(z=0;z<num;z++)
   {
			 Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(z));
			 Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(sptr[z].gram_address*1024L));
			 Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(sptr[z].image_format,sptr[z].linestride,sptr[z].image_height));
			 Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(sptr[z].filter,BORDER, BORDER,sptr[z].sizex,sptr[z].sizey));
#if defined(DISPLAY_RESOLUTION_WVGA)
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT_H(sptr[z].linestride>>10,sptr[z].image_height>>9));
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE_H(sptr[z].sizex>>9,sptr[z].sizey>>9));
#endif

   }
   Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
   Ft_Gpu_CoCmd_Swap(phost);
   Ft_App_Flush_Co_Buffer(phost);
   Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
}


static void rotate_around(ft_int16_t x, ft_int16_t y, ft_int16_t a)
{
  Ft_Gpu_CoCmd_LoadIdentity(phost);
  Ft_Gpu_CoCmd_Translate(phost,F16(x),F16(y));
  Ft_Gpu_CoCmd_Rotate(phost,a);
  Ft_Gpu_CoCmd_Translate(phost,F16(-x),F16(-y));
  Ft_Gpu_CoCmd_SetMatrix(phost);
}

static char *Array_Cycle_Name[9] = {0,"Eco Cold", "Normal", "Heavy","PermPress",  "Wool", "Bedding", "ActiveWear"};
char *Array_Cycle_Options[3] = {"Soil Level", "Temperature", "Spin Speed"};
char *Array_Menu_Options[3] = {"Child Lock", "Settings", "Start"};

char *BrightLevel[6] = {0,"Level1", "Level2", "Level3", "Level4", "Level5"};

char *SoundLevel[6] = {0,"Mute", "Level1", "Level2", "Level3", "Level4"};

typedef struct Item_Prop
{
		char *SoilLev;
		char *Temp;
		char *SpinSpeed;
}Item_Properties;

Item_Properties Item_Property[] =
{
		{"Light",  "Cold", "Low"},//Eco cold
		{"Normal", "Warm", "Medium"}, //Normal
		{"Heavy",  "Hot",  "High"}, // HEavy Duty
		{"Light",  "Warm", "Medium"},//Perm Press
		{"Normal", "Hot",  "Medium"},//Active Wear
		{"Heavy",  "Warm", "High"},//Bedding
		{"Heavy",  "Cold", "Low"},//Wool
};

typedef struct Bub
{
	ft_int16_t xOffset;
	ft_int16_t yOffset;
	ft_char8_t xDiff;
	ft_char8_t yDiff;

}Bubbles;

static ft_uint8_t ProcFinish;
ft_uint16_t WashWindow()
{
		ft_uint16_t BitmapTr=0,BubY=0,i=0,th=0,PwY=0,PwY1=0,PwY2=0;
		ft_int16_t RotX=0,RotY=0;
		ft_uint32_t ItCt=0;

#if defined(DISPLAY_RESOLUTION_WQVGA)
		ft_uint16_t pauseX=205, pauseY=190, progressCoverX2=435, progressCoverY1=134,progressCoverY2=178, processImageCX=225, processImageCY=85, processImageCSz=45, cycleStringX=40, cycleStringY=190, cycleDoneX=197, cycleDoneY=73, washCycleImgX=200, washSpinImgY=60;
		ft_uint8_t cycleFont=23, PwYLimit=61;
#elif defined(DISPLAY_RESOLUTION_WVGA)
		ft_uint16_t pauseX=372, pauseY=372, progressCoverX2=737, progressCoverY1=237,progressCoverY2=323, processImageCX=374, processImageCY=142, processImageCSz=80, cycleStringX=72, cycleStringY=340, cycleDoneX=340, cycleDoneY=130, washCycleImgX=330, washSpinImgY=250;
		ft_uint8_t cycleFont=29, PwYLimit=85;
#endif

#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
		ft_uint16_t pauseX=160, pauseY=280, progressCoverX2=320, progressCoverY1=134,progressCoverY2=178, processImageCX=225, processImageCY=85, processImageCSz=45, cycleStringX=40, cycleStringY=190, cycleDoneX=197, cycleDoneY=73, washCycleImgX=200, washSpinImgY=60;
		ft_uint8_t cycleFont=23, PwYLimit=61;
#endif
		//SYSTEMTIME st;
		static ft_uint8_t ProcessFlag;
		ft_uint8_t Read_tag=0,Sdflag=0,Sd1=0,Sd2=0,SoundFlag =0;

		Bubbles W_Bubble[40];

		Logo_Intial_setup(Wash_Window,7);

		if(MainFlag == 1)
		{
			TimeSet = 36;
#if defined(DISPLAY_RESOLUTION_WQVGA)
			Px=44;
#elif defined(DISPLAY_RESOLUTION_WVGA)
			Px=76;
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
			Px=44;
#endif

		}
		BubY = 110;

		for(i=0;i<40;i++)
		{
				W_Bubble[i].xOffset = ft_random(FT_DispWidth);
				W_Bubble[i].yOffset = ft_random(FT_DispHeight);

				W_Bubble[i].xDiff = ft_random(16) ;
				W_Bubble[i].yDiff = ft_random(8) ;
		}



		 do
		{

				Ft_Gpu_CoCmd_Dlstart(phost);        // start
				Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,255));
				Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));

				Read_tag = Read_Keys();


				/* Draw all the background bitmaps - balls with various resolutions */
				if(ProcessFlag == 2)
				{
					Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
					Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255 ,255 ,255)); // brown outside circle
					if(Pf ==0)
					{
							for(i = 0;i<25;i++)
							{
								Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(4));
								Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(W_Bubble[i].xOffset*16,W_Bubble[i].yOffset*16));

								W_Bubble[i].xOffset += W_Bubble[i].xDiff;
								W_Bubble[i].yOffset += W_Bubble[i].yDiff;

								//restrict the position of the background bitmaps to the visible region
								if(W_Bubble[i].xOffset < -80)
								{
									W_Bubble[i].xOffset = FT_DispWidth + 80;
								}
								else if(W_Bubble[i].xOffset > FT_DispWidth + 80)
								{
									W_Bubble[i].xOffset = -80;
								}

								if(W_Bubble[i].yOffset < -80)
								{
									W_Bubble[i].yOffset = FT_DispHeight + 80;
								}
								else if(W_Bubble[i].yOffset > FT_DispHeight + 80)
								{
									W_Bubble[i].yOffset = -80;
								}
						}
					}
				W_Bubble[ft_random(40)].xDiff = ft_random(16) - 6;
				W_Bubble[ft_random(40)].yDiff = ft_random(8) - 4;
				}
				// Progress bar Image
				Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));

				Ft_App_WrCoCmd_Buffer(phost,COLOR_A(180));
				Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128));
				Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,0,6,0));// backgnd tag -13

				Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
#ifdef DISPLAY_RESOLUTION_WQVGA
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(40,130,3,0));
#elif DISPLAY_RESOLUTION_WVGA
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(75,233,3,0));
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,130,3,0));
#endif
				// Pause Button

				Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(1));
				Ft_App_WrCoCmd_Buffer(phost, TAG('P'));
 				if(ItCt % 100 >0 && ItCt % 100 <50 && Pf == 1 && ProcessFlag<5)
						Ft_Gpu_CoCmd_Text(phost,pauseX,pauseY,31,0,"I I");
				else if(Pf == 0  && ProcessFlag<5)
						Ft_Gpu_CoCmd_Text(phost,pauseX,pauseY,31,0,"I I");



				Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(0));

				if(Read_tag == 'P')
				{
					if(Pf == 1)Pf = 0;
					else if(Pf ==0)	Pf =1;
				}

				//Progress bar cover up
				if(Px <progressCoverX2)
				{
				Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(129,178,218));
				Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(5*16));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(Px*16,progressCoverY1*16));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(progressCoverX2*16,progressCoverY2*16));

				}
				// Circle behind Process images
				Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
				Ft_App_WrCoCmd_Buffer(phost,COLOR_A(100));
				Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(processImageCSz*16));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(processImageCX*16,processImageCY*16));


				// Wash image
				Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));

				if(Pf == 0)// if Pause button not set
				{
						if(TimeSet > 30)
						{
								ProcessFlag = 1;
								Ft_Gpu_CoCmd_Text(phost,cycleStringX,cycleStringY,cycleFont,0,"Pre Wash");
								ProcFinish =1;
						}
						else if(TimeSet > 20 && TimeSet <= 30)
						{

								Ft_Gpu_Hal_Wr8(phost,REG_VOL_PB,0);
								if(Sdflag == 0)
								{
									 Play_Sound(0x56,SoundLev);
									 Sdflag = 1;
								}
								ProcessFlag = 2;
								Ft_Gpu_CoCmd_Text(phost,cycleStringX,cycleStringY,cycleFont,0,"Washing");
								ProcFinish =1;
						}
						else if(TimeSet <= 20 && TimeSet >10)
						{
								if(Sd1 == 0)
								{
									 Play_Sound(0x56,SoundLev);
									 Sd1 = 1;
								}
								ProcessFlag = 3;
								Ft_Gpu_CoCmd_Text(phost,cycleStringX,cycleStringY,cycleFont,0,"Rinsing");
								ProcFinish =1;
						}
						else if(TimeSet <= 10 && TimeSet >0)
						{
								if(Sd2 == 0)
								{
									 Play_Sound(0x56,SoundLev);
									 Sd2 = 1;
								}
								ProcessFlag = 4;
								Ft_Gpu_CoCmd_Text(phost,cycleStringX,cycleStringY,cycleFont,0,"Spinning");
								ProcFinish =1;
						}
						else if(TimeSet == 0)
						{
								ProcessFlag = 5;
								Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));
								if(ItCt %20 >0 && ItCt %20 < 10)
									Ft_Gpu_CoCmd_Text(phost,cycleDoneX,cycleDoneY,cycleFont,0,"DONE");

								MainFlag = 0;
								ProcFinish =2;
								Stflag = 0;
#if defined(DISPLAY_RESOLUTION_WQVGA)
								ButX = 330;
#elif defined(DISPLAY_RESOLUTION_WVGA)
								ButX = 592;
#endif
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
								ButX = 300;
#endif
						}

						{
							if(ItCt%25 == 0)
								BitmapTr = 0;
							else	if(ItCt%40 == 0)
								BitmapTr = 1;
							else	if(ItCt%60 == 0)
								BitmapTr = 2;
							else	if(ItCt%90 == 0)
								BitmapTr = 3;
						}
				}
				// Process Images

				Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
				// Pre Wash Image
				if(ProcessFlag == 1)// Images loading during pre wash stage
				{
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(washCycleImgX,PwY,1,0));
						if(PwY == PwYLimit)
						{
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(washCycleImgX,PwY1,0,0));
						}
						if(PwY1 == PwYLimit)
						{
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(washCycleImgX,PwY2,2,0));
						}

				}
				else if(ProcessFlag < 5)
				{
					if(ProcessFlag == 2 || ProcessFlag == 3)// washing and rinsing images
					{
#if defined(DISPLAY_RESOLUTION_WQVGA)
							RotX = 27;RotY = 27;
#elif defined(DISPLAY_RESOLUTION_WVGA)
							RotX = 48;RotY = 48;
#endif
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
							RotX = 27;RotY = 27;
#endif
							Ft_App_WrCoCmd_Buffer(phost,CMD_LOADIDENTITY);
							if(Pf == 0)
							rotate_around(RotX,RotY,th);
							Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(ProcessFlag-2));
							Ft_App_WrCoCmd_Buffer(phost,CELL(BitmapTr));
							Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(washCycleImgX*16,washSpinImgY*16));
							Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
					}

					else if(ProcessFlag == 4)// Spinning images
					{
						Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(ProcessFlag-2));
						Ft_App_WrCoCmd_Buffer(phost,CELL(BitmapTr));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(washCycleImgX*16,washSpinImgY*16));
					
					}
				}

				// Home Button
				Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(1));
				Ft_App_WrCoCmd_Buffer(phost, TAG('H'));
#if defined(DISPLAY_RESOLUTION_WQVGA)
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(430,10,5,0));
#elif defined(DISPLAY_RESOLUTION_WVGA)
				Ft_App_WrCoCmd_Buffer(phost, BITMAP_HANDLE(5));
				Ft_App_WrCoCmd_Buffer(phost, CELL(0));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth-Wash_Window[5].sizex-16)*16,16));
#endif
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(270,10,5,0));
#endif
				//fix the home button on the top right corner, position is the image width plus some space to the right
				Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(0));


				// Time remaining
#if defined(DISPLAY_RESOLUTION_WQVGA)
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
				Ft_Gpu_CoCmd_Number(phost, 40,  60, 31, OPT_CENTERX|OPT_CENTERY, 0);
				Ft_Gpu_CoCmd_Number(phost, 60,  60, 31, OPT_CENTERX|OPT_CENTERY, 0);
				Ft_Gpu_CoCmd_Text(phost, 75,  67, 21, OPT_CENTERX|OPT_CENTERY, "H");
				Ft_Gpu_CoCmd_Text(phost, 85,  57, 31, OPT_CENTERX|OPT_CENTERY, ":");

				if(TimeSet < 10)
				{
					Ft_Gpu_CoCmd_Number(phost,105,60,31,OPT_CENTERX|OPT_CENTERY,0);
					Ft_Gpu_CoCmd_Number(phost,128,60,31,OPT_CENTERX|OPT_CENTERY,TimeSet);
				}
				else{
					Ft_Gpu_CoCmd_Number(phost, 115, 60, 31, OPT_CENTERX|OPT_CENTERY, TimeSet);
				}

				Ft_Gpu_CoCmd_Text    (phost, 150, 67, 21, OPT_CENTERX|OPT_CENTERY, "Min"		);



				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
				Ft_Gpu_CoCmd_Text(phost,300,190,23,0,"Est. Finish Time");



				Ft_Gpu_CoCmd_Number(phost,315,212,25,0,0);
                Ft_Gpu_CoCmd_Number(phost,335,212,25,0,0);
				Ft_Gpu_CoCmd_Text(phost,355,210,25,0,":");
				if(TimeSet < 10)
				{
					Ft_Gpu_CoCmd_Number(phost, 370    , 212, 25, 0, 0	);
					Ft_Gpu_CoCmd_Number(phost, 390    , 212, 25, 0, TimeSet	);
					Ft_Gpu_CoCmd_Number(phost, 390    , 212, 25, 0, TimeSet	);
				}
				else{
					Ft_Gpu_CoCmd_Number(phost, 370    , 212, 25, 0, TimeSet	);
				}
#elif defined(DISPLAY_RESOLUTION_WVGA)
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
				Ft_Gpu_CoCmd_Number(phost, 74,  122, 31, OPT_CENTERX|OPT_CENTERY, 0);
				Ft_Gpu_CoCmd_Number(phost, 97,  122, 31, OPT_CENTERX|OPT_CENTERY, 0);
				Ft_Gpu_CoCmd_Text(phost, 115,  130, 28, OPT_CENTERX|OPT_CENTERY, "H");
				Ft_Gpu_CoCmd_Text(phost, 130,  118, 31, OPT_CENTERX|OPT_CENTERY, ":");

				if(TimeSet < 10)
				{
					Ft_Gpu_CoCmd_Number(phost,155,122,31,OPT_CENTERX|OPT_CENTERY,0);
					Ft_Gpu_CoCmd_Number(phost,179,123,31,OPT_CENTERX|OPT_CENTERY,TimeSet);
				}
				else{
					Ft_Gpu_CoCmd_Number(phost, 167, 123, 31, OPT_CENTERX|OPT_CENTERY, TimeSet);
				}

				Ft_Gpu_CoCmd_Text(phost, 209, 131, 28, OPT_CENTERX|OPT_CENTERY, "Min"		);

				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
				Ft_Gpu_CoCmd_Text(phost,598,355,28,OPT_CENTERX|OPT_CENTERY,"Est. Finish Time");


				Ft_Gpu_CoCmd_Number(phost,552,395,31,OPT_CENTERX|OPT_CENTERY,0);
                Ft_Gpu_CoCmd_Number(phost,576,395,31,OPT_CENTERX|OPT_CENTERY,0);
				Ft_Gpu_CoCmd_Text(phost,601,391,31,OPT_CENTERX|OPT_CENTERY,":");
				if(TimeSet < 10)
				{
					Ft_Gpu_CoCmd_Number(phost, 629, 395, 31, OPT_CENTERX|OPT_CENTERY, 0);
					Ft_Gpu_CoCmd_Number(phost, 654, 395, 31, OPT_CENTERX|OPT_CENTERY, TimeSet	);
					//Ft_Gpu_CoCmd_Number(phost, 390    , 212, 25, 0, TimeSet	);
				}
				else{
					Ft_Gpu_CoCmd_Number(phost, 642, 395, 31, OPT_CENTERX|OPT_CENTERY, TimeSet	);
				}
#endif

#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
				Ft_Gpu_CoCmd_Number(phost, 40,  60, 31, OPT_CENTERX|OPT_CENTERY, 0);
				Ft_Gpu_CoCmd_Number(phost, 60,  60, 31, OPT_CENTERX|OPT_CENTERY, 0);
				Ft_Gpu_CoCmd_Text(phost, 75,  67, 21, OPT_CENTERX|OPT_CENTERY, "H");
				Ft_Gpu_CoCmd_Text(phost, 85,  57, 31, OPT_CENTERX|OPT_CENTERY, ":");

				if(TimeSet < 10)
				{
					Ft_Gpu_CoCmd_Number(phost,105,60,31,OPT_CENTERX|OPT_CENTERY,0);
					Ft_Gpu_CoCmd_Number(phost,128,60,31,OPT_CENTERX|OPT_CENTERY,TimeSet);
				}
				else{
					Ft_Gpu_CoCmd_Number(phost, 115, 60, 31, OPT_CENTERX|OPT_CENTERY, TimeSet);
				}

				Ft_Gpu_CoCmd_Text    (phost, 150, 67, 21, OPT_CENTERX|OPT_CENTERY, "Min"		);



				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
				Ft_Gpu_CoCmd_Text(phost,170,190,23,0,"Est. Finish Time");



				Ft_Gpu_CoCmd_Number(phost,185,212,25,0,0);
                Ft_Gpu_CoCmd_Number(phost,200,212,25,0,0);
				Ft_Gpu_CoCmd_Text(phost,217,210,25,0,":");
				if(TimeSet < 10)
				{
					Ft_Gpu_CoCmd_Number(phost, 230    , 212, 25, 0, 0	);
					Ft_Gpu_CoCmd_Number(phost, 245    , 212, 25, 0, TimeSet	);
					//Ft_Gpu_CoCmd_Number(phost, 2    , 212, 25, 0, TimeSet	);
				}
				else{
					Ft_Gpu_CoCmd_Number(phost, 230    , 212, 25, 0, TimeSet	);
				}
#endif
				// play sound during every stage of process
				 if(TimeSet%10!=0)
                {
                  if(TimeSet % 2 == 0)
                  {
					  if(SoundFlag == 0)
					  {
							Play_Sound(0x50,SoundLev);
							SoundFlag = 1;
					  }
                  }
				  else
					  SoundFlag =0;
                }


				Ft_App_WrCoCmd_Buffer(phost,END());
				Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
				Ft_Gpu_CoCmd_Swap(phost);
				Ft_App_Flush_Co_Buffer(phost);

				Ft_Gpu_Hal_WaitCmdfifo_empty(phost);



				ItCt+=1;
				if(	ItCt%50	==	0	&& Pf == 0)	TimeSet--;
				if(	TimeSet	>	60	)	TimeSet =0;
				// Process rectangle cover up co ordinates
				if(	IterCount %50	==	0	&& Pf == 0)
#ifdef DISPLAY_RESOLUTION_WQVGA
					Px+=11;
#elif DISPLAY_RESOLUTION_WVGA
					Px+=19;

#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
					Px+=11;
#endif
				//Butterfly Line
				if(IterCount%15 == 0 && Pf==0 ){
					ButX+=1;
				}

				// Rinsing process - image roation
				if(ProcessFlag == 3)
				{
						if((ItCt%100 > 0) &&(ItCt%100 <50))
							th-=1000;
						if((ItCt%100 >= 50) &&(ItCt%100 <100))
							th+= 1000;
				}

				// Images loading during pre wash stage
				if(PwY  <PwYLimit)PwY+=1;
				if(PwY1 <PwYLimit && PwY == PwYLimit)PwY1+=1;
				if(PwY2 <PwYLimit && PwY1 == PwYLimit )PwY2+=1;
				IterCount+=1;

		}while(Read_tag !='H');
		OptionsSet = 1;

		return  Read_tag;
}


ft_uint8_t Ft_Gpu_Rom_Font_WH(ft_uint8_t Char,ft_uint8_t font)
{
	ft_uint32_t ptr,Wptr;
	ft_uint8_t Width=0;
	ptr = Ft_Gpu_Hal_Rd32(phost,ROMFONT_TABLEADDRESS);

	// read Width of the character
	Wptr = (ptr + (148L * (font- 16L)))+Char;	// (table starts at font 16)
	Width = Ft_Gpu_Hal_Rd8(phost,Wptr);
	return Width;
}

ft_uint8_t fontPixelHeight(ft_uint8_t font){

	ft_uint32_t ptr,hPtr;
	ft_uint8_t height=0;
	ptr = Ft_Gpu_Hal_Rd32(phost,ROMFONT_TABLEADDRESS);

	// the height is at the 140th byte
	hPtr = (ptr + (148 * (font- 16)))+140;	// (table starts at font 16)
	height = Ft_Gpu_Hal_Rd8(phost,hPtr);
	return height;
}

ft_uint16_t stringPixelWidth(const ft_uchar8_t* text,ft_uint8_t font){
	ft_char8_t tempChar;
	ft_uint16_t length=0, index;
	if(text==NULL){
		return 0;
	}

	if(text[0]=='\0'){
		return 0;
	}

	index=0;
	tempChar=text[index];
	while(tempChar!='\0'){
		length+=Ft_Gpu_Rom_Font_WH(tempChar, font);
		tempChar=text[++index];
	}

	return length;
}



static ft_uint8_t BrightKey,VolKey;

ft_uint8_t Settings()
{
	//new
#ifdef DISPLAY_RESOLUTION_WQVGA
	ft_uint16_t SettingBlockHeight=30, SettingBlockWidth=50, SettingBlockX=115, SettingBlockY=25, SettingBlockGap=10;
	static ft_uint16_t SettingBackgroundHeight = 215, DisplayXPos=60, DisplayYPos=80, SoundXPos=60, SoundYPos=145;
	ft_uint8_t TextFont=23, SettingFont=18;
#elif DISPLAY_RESOLUTION_WVGA
	ft_uint16_t SettingBlockHeight=50, SettingBlockWidth=80, SettingBlockX=200, SettingBlockY=100, SettingBlockGap=10;
	static ft_uint16_t SettingBackgroundHeight, DisplayXPos=100, DisplayYPos=100, SoundXPos=100, SoundYPos=200;
	ft_uint8_t TextFont=30, SettingFont=23;
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
	ft_uint16_t SettingBlockHeight=30, SettingBlockWidth=50, SettingBlockX=30, SettingBlockY=35, SettingBlockGap=10;
	static ft_uint16_t SettingBackgroundHeight = 400, DisplayXPos=80, DisplayYPos=80, SoundXPos=180, SoundYPos=80;
	ft_uint8_t TextFont=23, SettingFont=18;
#endif

	static ft_uint16_t SettingBackgroundWidth;
	static ft_uint8_t firstTime=1;
	ft_uint8_t Read_tag=0,i=0,SoundFlag=0;
	ft_uint16_t Rx1 =0,Ry1=0,Rx2=0,Ry2=0, startingX, startingY;
        char buffer[10];

	if(firstTime){
		SettingBackgroundWidth = SettingBlockWidth * 8 + SettingBlockGap * 6;;   //5 setting blocks and 6 gaps between the setting blocks
		//SettingBackgroundWidth = FT_DispWidth;
		//SettingBackgroundHeight = SettingBlockHeight*5 + SettingBlockGap*6;
		SettingBackgroundHeight = 380;
		DisplayYPos = SettingBlockY + 3*SettingBlockHeight/2 + SettingBlockGap * 2;  //align the "Display:" text with the setting blocks
		SoundYPos = SettingBlockY + 7*SettingBlockHeight/2 + SettingBlockGap*4; //align the "Sound:" text with the setting blocks.
	}

		if(BrightKey == 0)BrightKey = 5;
		if(VolKey == 0)VolKey = 10;
	do{
			Read_tag = Read_Keys();
			Ft_Gpu_CoCmd_Dlstart(phost);

			Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,255));
			Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));

			Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(180));

			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128));
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128));

			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,0,9,0));// backgnd tag -8

			Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());

			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,200,255));

			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
			Ft_App_WrCoCmd_Buffer(phost,TAG('V'));
			Ft_Gpu_CoCmd_Text(phost,DisplayXPos,DisplayYPos,TextFont,OPT_CENTERX|OPT_CENTERY,"Display:");

			Ft_App_WrCoCmd_Buffer(phost,TAG('B'));
			Ft_Gpu_CoCmd_Text(phost,SoundXPos,SoundYPos,TextFont,OPT_CENTERX|OPT_CENTERY,"Sound:");

			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

			//rectangle enclosing the selectable setting blocks
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
			//Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(4*16));
			Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(16));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(100));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(SettingBlockX*16, SettingBlockY*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(SettingBackgroundWidth*16, SettingBackgroundHeight*16));

			//Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
			Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(4*16));

			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));

			// Brightness
			startingX = SettingBlockX + SettingBlockGap;
			startingY = SettingBlockY + SettingBlockHeight + SettingBlockGap * 2;
			for(i =1;i<6;i++)
			{
					if(i==1)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(177,176,180));

					if(i==2)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(122,154,171));

					if(i==3)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(70,149,186));

					if(i==4)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(28,140,202));

					if(i==5)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(135,183,71));

					if(i == BrightKey)
					{
						Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,BrightNess);
						Ft_App_WrCoCmd_Buffer(phost,TAG(i));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((startingX-3)*16, (startingY-3)*16));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((startingX+SettingBlockWidth+3)*16, (startingY+SettingBlockHeight+3)*16));
					}
					else
					{
						Ft_App_WrCoCmd_Buffer(phost,TAG(i));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((startingX)*16, (startingY)*16));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((startingX+SettingBlockWidth)*16, (startingY+SettingBlockHeight)*16));
					}
					startingX += (SettingBlockWidth+SettingBlockGap);
			}

			startingX = SettingBlockX + SettingBlockWidth/2 + SettingBlockGap;
			startingY = SettingBlockY + 3*SettingBlockHeight/2 + SettingBlockGap * 2;
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));

			for(i =1;i<6;i++)
			{
					Ft_App_WrCoCmd_Buffer(phost,TAG(i));
					Ft_Gpu_CoCmd_Text(phost,startingX,startingY,SettingFont,OPT_CENTERX|OPT_CENTERY,BrightLevel[i]);
					startingX += (SettingBlockWidth + SettingBlockGap);
			}


			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

			Ft_App_WrCoCmd_Buffer(phost,END());

			// Volume

			Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
			Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(4*16));

			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));

			startingX = SettingBlockX + SettingBlockGap;
			startingY = SettingBlockY + SettingBlockHeight*3 + SettingBlockGap*4;

			for(i =1;i<6;i++)
			{
					if(i==1)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(177,176,180));

					if(i==2)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(122,154,171));

					if(i==3)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(70,149,186));

					if(i==4)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(28,140,202));

					if(i==5)Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(135,183,71));

					if(i == VolKey-5)
					{
						//Play_Sound(0x07,SoundLev);
						if(SoundFlag == 0)
						{
							Play_Sound(0x56,SoundLev);
							SoundFlag =1;
						}
						Ft_App_WrCoCmd_Buffer(phost,TAG(i+5));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((startingX-3)*16, (startingY-3)*16));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((startingX+SettingBlockWidth+3)*16, (startingY+SettingBlockHeight+3)*16));

					}
					else
					{
						Ft_App_WrCoCmd_Buffer(phost,TAG(i+5));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((startingX)*16, (startingY)*16));
						Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((startingX+SettingBlockWidth)*16, (startingY+SettingBlockHeight)*16));

					}
					startingX += (SettingBlockWidth+SettingBlockGap);
			}

			startingX = SettingBlockX + SettingBlockGap + SettingBlockWidth/2;
			startingY = SettingBlockY + 7*SettingBlockHeight/2 + SettingBlockGap*4;

			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));

			for(i =1;i<6;i++)
			{
					Ft_App_WrCoCmd_Buffer(phost,TAG(i+5));
					Ft_Gpu_CoCmd_Text(phost,startingX,startingY,SettingFont,OPT_CENTERX|OPT_CENTERY,SoundLevel[i]);
					startingX += (SettingBlockWidth + SettingBlockGap);
			}

			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
			Ft_App_WrCoCmd_Buffer(phost,END());

			if((Read_tag >0) && (Read_tag <6) )
			{
					BrightKey =Read_tag;
			}
			if((Read_tag >5) && (Read_tag <11) )
			{
					VolKey =Read_tag;
					SoundFlag = 0;
			}
			if( Read_tag >0 && Read_tag < 6 )
				BrightNess = Read_tag*128/5;
			if( Read_tag >5 && Read_tag < 11 )
				SoundLev = (Read_tag-6)*63;

			// Home Button
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
			Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(1));
			Ft_App_WrCoCmd_Buffer(phost, TAG('H'));
#ifdef DISPLAY_RESOLUTION_WQVGA
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(FT_DispWidth - Main_Icons[8].sizex - 10,10,8,0));
#elif DISPLAY_RESOLUTION_WVGA
			Ft_App_WrCoCmd_Buffer(phost, BITMAP_HANDLE(8));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth - Main_Icons[8].sizex - 10)*16,10*16));
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(FT_DispWidth - Main_Icons[8].sizex - 10,10,8,0));
#endif
			Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(0));

			Ft_App_WrCoCmd_Buffer(phost,END());

			Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
			Ft_Gpu_CoCmd_Swap(phost);
			Ft_App_Flush_Co_Buffer(phost);
			Ft_Gpu_Hal_WaitCmdfifo_empty(phost);


	}while(Read_tag != 'H');

	return Read_tag;

}


static  ft_uint8_t MenuFlag;
ft_uint8_t MainWindow()
{
#if defined(DISPLAY_RESOLUTION_WQVGA) || defined(DISPLAY_RESOLUTION_QVGA)
	ft_uint16_t scrollPaneXMark=120,Ty=0, washingTypeXoffset=60, washingTypeTextFont=23, washingTypeTextSpace=50;
	ft_uint16_t centerPaneX0=140, centerPaneY0=25, centerPaneX1=320, centerPaneY1=240, optionsTextX=220, optionsTexty=32, optionsTextFont=23, centerPaneIconsYOffset=60, centerPaneIconsXOffset=150, centerPaneTextXOffset=240 , centerPaneTextFont=18, centerPaneTextSpace=60, centerPaneLineYOffset=55, centerPaneLineX0=140, centerPaneLineX1=320, centerPaneLineSpacing=60;
	ft_uint16_t itemPropYOffset=90, itemPropXOffset=240, itemPropFont=29, itemPropSpacing=60, soilLevX=150, soilLevY=60, tempLevPointSz=8, tempLevPointX=170, tempLevPointY=150, tempLevLineWidth=1, tempLevLineX=170, tempLevLineY0=145, tempLevLineY1=135, spinSpeedTextX=150, spinSpeedTexty=180;
#elif defined(DISPLAY_RESOLUTION_WVGA)
	ft_uint16_t scrollPaneXMark=250,Ty=0, washingTypeXoffset=118,washingTypeTextFont=29, washingTypeTextSpace=85;
	ft_uint16_t centerPaneX0=250, centerPaneY0=45, centerPaneX1=550, centerPaneY1=425, optionsTextX=370, optionsTexty=60, optionsTextFont=30, centerPaneIconsYOffset=90, centerPaneIconsXOffset=285, centerPaneTextXOffset=427 , centerPaneTextFont=29, centerPaneTextSpace=120, centerPaneLineYOffset=80, centerPaneLineX0=250, centerPaneLineX1=540, centerPaneLineSpacing=118;
	ft_uint16_t itemPropYOffset=140, itemPropXOffset=427, itemPropFont=30, itemPropSpacing=118, soilLevX=285, soilLevY=90, tempLevPointSz=8, tempLevPointX=320, tempLevPointY=265, tempLevLineWidth=2, tempLevLineX=320, tempLevLineY0=265, tempLevLineY1=235, spinSpeedTextX=285, spinSpeedTexty=334;
#endif

#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
	ft_uint16_t scrollPaneXMark=300,Ty=0, washingTypeXoffset=60, washingTypeTextFont=23, washingTypeTextSpace=50;
	ft_uint16_t centerPaneX0=140, centerPaneY0=25, centerPaneX1=320, centerPaneY1=240, optionsTextX=220, optionsTexty=32, optionsTextFont=23, centerPaneIconsYOffset=60, centerPaneIconsXOffset=150, centerPaneTextXOffset=240 , centerPaneTextFont=18, centerPaneTextSpace=60, centerPaneLineYOffset=55, centerPaneLineX0=140, centerPaneLineX1=320, centerPaneLineSpacing=60;
	ft_uint16_t itemPropYOffset=90, itemPropXOffset=240, itemPropFont=29, itemPropSpacing=60, soilLevX=150, soilLevY=60, tempLevPointSz=8, tempLevPointX=170, tempLevPointY=150, tempLevLineWidth=1, tempLevLineX=170, tempLevLineY0=145, tempLevLineY1=135, spinSpeedTextX=150, spinSpeedTexty=180;
#endif
	ft_uint16_t By=0,Ly=0,th=0,Addth=0;
	ft_uint8_t PrevTag=0,Read_Tag=0,temp=0,dg_count=0,LockFlag=0,PlayFlag=0,Opt=0,i=0;
	ft_int16_t sy = 0,	drag = 0;

	static ft_uint8_t j;
	Ty = 0;
#if defined(FT900_PLATFORM)
	scroller_init(160);
#else
	scroller_init(100);
#endif


	Logo_Intial_setup(Main_Icons,10);

	do
	{
			Ft_Gpu_Hal_Wr8(phost,REG_VOL_SOUND,0);
			Ft_Gpu_Hal_Wr8(phost,REG_VOL_PB,0);

			//Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,100);
			sy =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);

			Read_Tag = Read_Keys();

#if 0
			if(sy != -32768 )	// touch
			{
				  //printf("Y: %d", sy);
				  dg_count++;
				  temp = key_pressed;
			}
			/*Move into the particular frame based on dragdt now 30pixels*/
			if(sy==-32768)
			{
				  dg_count = 0;
				  drag = scroller.base>>4;
			}
			/*if tag in but still pendown take a scroller basevalue*/
			else if(dg_count>15)
			{
				  Read_Tag = 0;
				  temp = key_pressed;
				  drag = scroller.base>>4;
			}
			if(drag>= 135)
			{
				drag = 135;
			}

			if(temp == 101/* ||((temp > 0) &&(temp < 8)))*/ && drag <= 135)
			{
				scroller_run();
			}
#endif
			drag = scroller_run(key_pressed);


			Ft_Gpu_CoCmd_Dlstart(phost);

			Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(0,0,255));
			Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));


		    // BackGround Image
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));

			//Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(180));
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128));
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128));

			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(0,0,9,0));// backgnd tag -8

			Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());


			// Draw rectanglw with zero alpha value to have scrolling option on left pane
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(0));
			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
			Ft_App_WrCoCmd_Buffer(phost,TAG(101));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0*16, 0*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(scrollPaneXMark*16, FT_DispHeight*16));

			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
			Ft_App_WrCoCmd_Buffer(phost,END());
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));

			Ty = 30;

			Ty = Ty-drag;
			// Options on the left pane

			//draw the washing type
			for(i =1; i<8;i++)
			{
					if(i == j)
					{
							Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
					}
					else
							Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,200,255));

					Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
					Ft_App_WrCoCmd_Buffer(phost,TAG(i));

					Ft_Gpu_CoCmd_Text(phost,washingTypeXoffset,Ty,washingTypeTextFont,OPT_CENTERX|OPT_CENTERY, Array_Cycle_Name[i]);
					Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

					Ty += washingTypeTextSpace;

			}


			// Draw rectangle in the centre to show the options
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));

			Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(4*16));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(100));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));

			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(centerPaneX0*16, centerPaneY0*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(centerPaneX1*16, centerPaneY1*16));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
			Ft_Gpu_CoCmd_Text(phost,optionsTextX,optionsTexty,optionsTextFont,OPT_CENTERX|OPT_CENTERY, "Options");


			// Display icons in vertical order
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));

			By = centerPaneIconsYOffset;

			for(i=0;i<3;i++)
			{
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
				if(i<2){
				Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(i));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(centerPaneIconsXOffset*16,(By)*16));
				}

				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
				Ft_Gpu_CoCmd_Text(phost,centerPaneTextXOffset,By+10,centerPaneTextFont,OPT_CENTERX|OPT_CENTERY, Array_Cycle_Options[i]);
				By+=centerPaneTextSpace;
			}

			// Lines draw after each icon
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
			Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(100));
			Ly = centerPaneLineYOffset;

			for(i=0;i<3;i++)
			{
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(centerPaneLineX0*16, Ly*16));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(centerPaneLineX1*16, Ly*16));
				Ly += 118;
			}
			Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
			By = itemPropYOffset;
			if(key_pressed > 0 && key_pressed < 9 )
			{

				if(ProcFinish == 0 || ProcFinish ==2|| TimeSet ==0)
				{
				Tagval =key_pressed;
				OptionsSet =1;
				temp =0;
				j =key_pressed;

				}
			}

			if(Tagval !=0)
			{
					for(i=0;	i<3;	i++)
					{
						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,255));
						if(i==0)
							Ft_Gpu_CoCmd_Text(phost,itemPropXOffset,By,itemPropFont,OPT_CENTERX|OPT_CENTERY, Item_Property[Tagval-1].SoilLev);
						if(i==1)
							Ft_Gpu_CoCmd_Text(phost,itemPropXOffset,By,itemPropFont,OPT_CENTERX|OPT_CENTERY, Item_Property[Tagval-1].Temp);
						if(i==2)
							Ft_Gpu_CoCmd_Text(phost,itemPropXOffset,By,itemPropFont,OPT_CENTERX|OPT_CENTERY, Item_Property[Tagval-1].SpinSpeed);
						By+=itemPropSpacing;

					}

			}

			// for soil icon
			if(Item_Property[Tagval-1].SoilLev == "Light")
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(200,200,200));
			else if(Item_Property[Tagval-1].SoilLev == "Normal" )
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(150,150,150));
			else if(Item_Property[Tagval-1].SoilLev == "Heavy")
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(80,80,80));
			else
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
			 Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
			 Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(soilLevX,soilLevY,0,0));


			// for temp icon - draw point and line inside icon
			if(Item_Property[Tagval-1].Temp == "Cold")
			 //if(Item_Property[Tagval-1].Temp[0] == 'C')
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,255));//blue
			else if(Item_Property[Tagval-1].Temp == "Warm" )
			// else if(Item_Property[Tagval-1].Temp[0] == 'W' )
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,140,0));//orange
			else if(Item_Property[Tagval-1].Temp == "Hot")
			//else if(Item_Property[Tagval-1].Temp[0] == 'H')
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));//red

			///this is the filling of the temperature icon
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
			Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(tempLevPointSz*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(tempLevPointX*16, tempLevPointY*16));
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
			Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(tempLevLineWidth*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(tempLevLineX*16, tempLevLineY0*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(tempLevLineX*16, tempLevLineY1*16));

			// Spin Speed rotation
			if(Item_Property[Tagval-1].SpinSpeed == "Low")
				Addth = 1000;
			else if(Item_Property[Tagval-1].SpinSpeed == "Medium")
				Addth = 2000;
			else if(Item_Property[Tagval-1].SpinSpeed == "High")
				Addth = 3000;
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,255));
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
			Ft_App_WrCoCmd_Buffer(phost,CMD_LOADIDENTITY);
			//rotate_around(20,20,th);	  //Main_Icons[8].sizex
			rotate_around(Main_Icons[2].sizex/2,Main_Icons[2].sizey/2,th);
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(spinSpeedTextX,spinSpeedTexty,2,0));
			Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());


			// Right side Pane

			// Estimated TIme
			if(Stflag == 0)
				TimeSet = 0;
			//else TimeSet = 0;
			if(TimeSet > 36)TimeSet = 0;
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,200,255));
#if defined(DISPLAY_RESOLUTION_WQVGA)
			Ft_Gpu_CoCmd_Text(phost,400,30,18,OPT_CENTERX|OPT_CENTERY,"Estimated Time");
			Ft_Gpu_CoCmd_Number(phost,355,60,31,OPT_CENTERX|OPT_CENTERY,0);
			Ft_Gpu_CoCmd_Text(phost,372,67,21,OPT_CENTERX|OPT_CENTERY,"H");
#elif defined(DISPLAY_RESOLUTION_WVGA)
			Ft_Gpu_CoCmd_Text(phost,684,37,28,OPT_CENTERX|OPT_CENTERY,"Estimated Time");
			Ft_Gpu_CoCmd_Number(phost,622,82,31,OPT_CENTERX|OPT_CENTERY,0);
			Ft_Gpu_CoCmd_Text(phost,641,88,27,OPT_CENTERX|OPT_CENTERY,"H");
#endif

#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
			Ft_Gpu_CoCmd_Text(phost,60,400,18,OPT_CENTERX|OPT_CENTERY,"Estimated Time");
			Ft_Gpu_CoCmd_Number(phost,10,430,31,OPT_CENTERX|OPT_CENTERY,0);
			Ft_Gpu_CoCmd_Text(phost,30,435,21,OPT_CENTERX|OPT_CENTERY,"H");
#endif

#if defined(DISPLAY_RESOLUTION_WQVGA)
			if(TimeSet < 10)
			{
				Ft_Gpu_CoCmd_Number(phost,395,60,31,OPT_CENTERX|OPT_CENTERY,0);
				Ft_Gpu_CoCmd_Number(phost,415,60,31,OPT_CENTERX|OPT_CENTERY,TimeSet);
			}
			else
				Ft_Gpu_CoCmd_Number(phost,405,60,31,OPT_CENTERX|OPT_CENTERY,TimeSet);
			Ft_Gpu_CoCmd_Text(phost,442,67,21,OPT_CENTERX|OPT_CENTERY,"Min");
#elif defined(DISPLAY_RESOLUTION_WVGA)
			if(TimeSet < 10)
			{
				Ft_Gpu_CoCmd_Number(phost,673,82,31,OPT_CENTERX|OPT_CENTERY,0);
				Ft_Gpu_CoCmd_Number(phost,697,82,31,OPT_CENTERX|OPT_CENTERY,TimeSet);
			}
			else
				Ft_Gpu_CoCmd_Number(phost,685,82,31,OPT_CENTERX|OPT_CENTERY,TimeSet);
			Ft_Gpu_CoCmd_Text(phost,725,88,27,OPT_CENTERX|OPT_CENTERY,"Min");
#endif
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
			if(TimeSet < 10)
			{
				Ft_Gpu_CoCmd_Number(phost,50,430,31,OPT_CENTERX|OPT_CENTERY,0);
				Ft_Gpu_CoCmd_Number(phost,75,430,31,OPT_CENTERX|OPT_CENTERY,TimeSet);
			}
			else
				Ft_Gpu_CoCmd_Number(phost,60,430,31,OPT_CENTERX|OPT_CENTERY,TimeSet);
				Ft_Gpu_CoCmd_Text(phost,100,435,21,OPT_CENTERX|OPT_CENTERY,"Min");
#endif
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(LINES));
			Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));

#if defined(DISPLAY_RESOLUTION_WQVGA)
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(92,148,212));
			//Blue line below butterfly
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(340*16, 90*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(460*16, 90*16));

			//Orange line below butterfly
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,140,0));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(340*16, 90*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(ButX*16, 90*16));
#elif defined(DISPLAY_RESOLUTION_WVGA)
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(92,148,212));
			//Blue line below butterfly
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(592*16, 129*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(760*16, 129*16));

			//Orange line below butterfly
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,140,0));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(592*16, 129*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(ButX*16, 129*16));
#endif
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(92,148,212));
			//Blue line below butterfly
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0*16, 470*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(320*16, 470*16));

			//Orange line below butterfly
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,140,0));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0*16, 470*16));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(ButX*16, 470*16));
#endif
			//Butterfly
			Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
#if defined(DISPLAY_RESOLUTION_WQVGA)
			if(ButX > 460)ButX =460;
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(ButX,75 ,3,0));
#elif defined(DISPLAY_RESOLUTION_WVGA)
			if(ButX > 765)ButX =765;
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(3));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(ButX*16,105*16));
#endif
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
			if(ButX > 320)ButX =320;
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(ButX,450 ,3,0));
#endif


			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));



			//Lock & Unlock Buton
			if(Read_Tag == 'U')LockFlag = 0;

			{
					Ft_App_WrCoCmd_Buffer(phost,TAG('L'));
#ifdef DISPLAY_RESOLUTION_WQVGA
					Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(340,105 ,4,0));
#elif DISPLAY_RESOLUTION_WVGA
					Ft_App_WrCoCmd_Buffer(phost, BITMAP_HANDLE(4));
					Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(600*16,163*16));
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
					Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(160,340 ,4,0));
#endif
			}

			if(Read_Tag == 'L')LockFlag = 1;



			// Settings Button
			Ft_App_WrCoCmd_Buffer(phost,TAG('S'));
#ifdef DISPLAY_RESOLUTION_WQVGA
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(340,145 ,6,0));
#elif DISPLAY_RESOLUTION_WVGA
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(6));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(600*16,249*16));
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(160,380 ,6,0));
#endif
			// Start Button
			Ft_App_WrCoCmd_Buffer(phost,TAG('P'));
#ifdef DISPLAY_RESOLUTION_WQVGA
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(340,185 ,7,0));
#elif DISPLAY_RESOLUTION_WVGA
			Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(7));
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(600*16,333*16));
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
			Ft_App_WrCoCmd_Buffer(phost,VERTEX2II(160,420 ,7,0));
#endif
			Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

			if(Read_Tag =='S')
					Settings();

			if(BrightNess == 0)BrightNess =100;
		    Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,BrightNess);// start

			if(TimeSet == 0) ProcFinish=2;
			if((Read_Tag == 'P'))
			{
				PlayFlag = 1;
				Opt=0;
				if(ProcFinish == 2)
				{
				TimeSet = 36;

#if defined(DISPLAY_RESOLUTION_WQVGA)
			Px=44;
			ButX =330;
#elif defined(DISPLAY_RESOLUTION_WVGA)
			Px =76;
			ButX =592;
#endif
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
			Px=44;
			ButX =280;
#endif
				}
			}
			if(PlayFlag ==1)
			{
					if(	OptionsSet == 1)
					{
							Stflag = 1;
							OptionsSet = 2;
					}
					else if(OptionsSet != 1 && Opt !=1)
					{
							if((IterCount % 100 > 0)&&( IterCount % 100 < 100))
							{
#ifdef DISPLAY_RESOLUTION_WQVGA
									Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));
									Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(110*16,115*16));
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(375*16,140*16));
									Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
									Ft_Gpu_CoCmd_Text(phost,120,125,27,OPT_CENTERY,"Please Select a Washing Mode");
#elif DISPLAY_RESOLUTION_WVGA
									Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));
									Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(255*16,255*16));
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(545*16,300*16));
									Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
									Ft_Gpu_CoCmd_Text(phost,400,278,28,OPT_CENTERX|OPT_CENTERY,"Please Select a Washing Mode");
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
									Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));
									Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(30*16,115*16));
									Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(280*16,140*16));
									Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
									Ft_Gpu_CoCmd_Text(phost,40,125,27,OPT_CENTERY,"Please Select a Washing Mode");
#endif
									if(IterCount % 100 == 99)
									{
										Opt=1;
										PlayFlag=0;
									}
							}
					}
			}

			if(temp >= 50)temp =0;
			Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
#ifdef DISPLAY_RESOLUTION_WQVGA
			By = 115;
			for(i=0;i<3;i++)
			{
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,200,255));
				Ft_Gpu_CoCmd_Text(phost,380,By,18,0,Array_Menu_Options[i]);
				By+=40;
#elif DISPLAY_RESOLUTION_WVGA
			By = 182;
			for(i=0;i<3;i++)
			{
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,200,255));
				Ft_Gpu_CoCmd_Text(phost,666,By,28,0,Array_Menu_Options[i]);
				By+=75;
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
			By = 340;
			for(i=0;i<3;i++)
			{
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,200,255));
				Ft_Gpu_CoCmd_Text(phost,210,By+9,18,0,Array_Menu_Options[i]);
				By+=40;
#endif
			}
			if(LockFlag ==1)
			{
					Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
					Ft_App_WrCoCmd_Buffer(phost,COLOR_A(180));
					Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
					Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,FT_DispHeight*16));
					Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));
					Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
					Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());

					Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
#if defined(SAMAPP_DISPLAY_WQVGA) || defined(SAMAPP_DISPLAY_QVGA)
					Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128) );
					Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128) );
#endif
					Ft_App_WrCoCmd_Buffer(phost,TAG('U'));
					Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(4));
					Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth/2)*16,(FT_DispHeight/2)*16));

					Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
			}

			if(MainFlag == 0)
#if defined(DISPLAY_RESOLUTION_WQVGA)
			Px=44;
#elif defined(DISPLAY_RESOLUTION_WVGA)
			Px=76;
#endif
#if defined(DISPLAY_RESOLUTION_HVGA_PORTRAIT)
			Px=44;
#endif

			Ft_App_WrCoCmd_Buffer(phost,END());


			Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
			Ft_Gpu_CoCmd_Swap(phost);
			Ft_App_Flush_Co_Buffer(phost);
			Ft_Gpu_Hal_WaitCmdfifo_empty(phost);



			th+=Addth;
			if(IterCount%15 == 0 && Stflag == 1 && Pf ==0)
			{
					ButX+=1;
			}
			if(IterCount%50 == 0 && Pf ==0&& Stflag == 1 )
				TimeSet--;
			if(	IterCount %50	==	0 && Pf ==0	)
#ifdef DISPLAY_RESOLUTION_WQVGA
					Px+=11;
#elif DISPLAY_RESOLUTION_WVGA
					Px+=19;
#endif
#ifdef DISPLAY_RESOLUTION_HVGA_PORTRAIT
					Px+=11;
#endif

			if(temp >=1 )temp++;

			IterCount+=1;
	}while(OptionsSet !=2 );//(Read_Tag!= 'P');
	MainFlag ++;
	return Read_Tag;
}

#if defined(MSVC_PLATFORM) || defined(FT900_PLATFORM)
/* Main entry point */
ft_int32_t main(ft_int32_t argc,ft_char8_t *argv[])
#endif
#if defined(ARDUINO_PLATFORM) || defined(MSVC_FT800EMU)
ft_void_t setup()
#endif
{
	ft_uint8_t Val;
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
	char *path = "..\\..\\..\\..\\Test";
#endif

	 ft_uint8_t chipid;
#ifdef FT900_PLATFORM
	FT900_Config();
#endif
	Ft_Gpu_HalInit_t halinit;
	halinit.TotalChannelNum = 1;
	Ft_Gpu_Hal_Init(&halinit);
	host.hal_config.channel_no = 0;
	host.hal_config.pdn_pin_no = FT800_PD_N;
	host.hal_config.spi_cs_pin_no = FT800_SEL_PIN;
    #if defined(MSVC_PLATFORM) && defined(MSVC_PLATFORM_SPI_LIBMPSSE)
	    host.hal_config.spi_clockrate_khz = 12000; //in KHz
    #elif defined(MSVC_PLATFORM) && defined(MSVC_PLATFORM_SPI_LIBFT4222)
    host.hal_config.spi_clockrate_khz = 20000; //in KHz
    #endif
#ifdef ARDUINO_PLATFORM_SPI
	host.hal_config.spi_clockrate_khz = 4000; //in KHz
#endif
    Ft_Gpu_Hal_Open(&host);
	phost = &host;
    Ft_BootupConfig();

#if ((defined FT900_PLATFORM) || defined(MSVC_PLATFORM))
	printf("\n reg_touch_rz =0x%x ", Ft_Gpu_Hal_Rd16(phost, REG_TOUCH_RZ));
	printf("\n reg_touch_rzthresh =0x%x ", Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_RZTHRESH));
    printf("\n reg_touch_tag_xy=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG_XY));
	printf("\n reg_touch_tag=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG));
#endif
    /*It is optional to clear the screen here*/
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
	Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);
#if defined(MSVC_PLATFORM) || defined(MSVC_FT800EMU)
  chdir(path);
#endif

 Info();
 Logo_Intial_setup(Main_Icons,7);
  while(1)
  {

	 Val =  MainWindow();
	 if(Val == 'P')
	 {
		   Play_Sound(0x56,SoundLev);
			WashWindow();
	 }
  }
    Ft_Gpu_Hal_Close(phost);
    Ft_Gpu_Hal_DeInit();
#ifdef MSVC_PLATFORM
	return 0;
#endif
}

void loop()
{
}



/* Nothing beyond this */













